      SUBROUTINE boundary1(flat,pres,ngrid,rho,vx,vy,vz,
     .     g,c,gamma,dx)
!     
!     compute boundary values of the hydro quantities with periodic
!     boundary conditions
!     

      IMPLICIT NONE             

      INTEGER :: ngrid
      REAL*8, DIMENSION(ngrid) :: pres,rho,vx,vy,vz,g,c
      REAL*8 :: gamma,dx

!     
!     parameters

!     
!     local variables


      INTEGER :: j
      REAL*8 :: pbar(5),rhobar(5),vxbar(5),vybar(5),vzbar(5),
     .     gbar(5),cbar(2)
      REAL*8 :: rhojl(2),pjl(2),vxjl(2),vyjl(2),vzjl(2),cjl(2),gjl(2)
      REAL*8 :: rhojr(2),pjr(2),vxjr(2),vyjr(2),vzjr(2),cjr(2),gjr(2)
      REAL*8 :: deltarho(2),deltap(2),deltavx(2),deltavy(2),
     .     deltavz(2),deltac(2),deltag(2)
      REAL*8 :: rho6(2),p6(2),vx6(2),vy6(2),vz6(2),c6(2),g6(2)
      REAL*8 :: addp,addrho,flat
!     
      common/bound/pbar,rhobar,vxbar,vybar,vzbar,gbar,cbar
      common/jleft/rhojl,pjl,vxjl,vyjl,vzjl,cjl,gjl
      common/jright/rhojr,pjr,vxjr,vyjr,vzjr,cjr,gjr
      common/delta/deltarho,deltap,deltavx,deltavy,deltavz,deltac,deltag
      common/a6/rho6,p6,vx6,vy6,vz6,c6,g6
!     
!     compute the interpolation functions for the zone 1
!     
      do j=1,5
         pbar(j)=pres(mod(ngrid+j-3,ngrid)+1)
         rhobar(j)=rho(mod(ngrid+j-3,ngrid)+1)
         vxbar(j)=vx(mod(ngrid+j-3,ngrid)+1)
         vybar(j)=vy(mod(ngrid+j-3,ngrid)+1)
         vzbar(j)=vz(mod(ngrid+j-3,ngrid)+1)
         gbar(j)=g(mod(ngrid+j-3,ngrid)+1)
      enddo
      cbar(1)=c(ngrid)
      cbar(2)=c(1)
!     
      addp=abs(pbar(4)-pbar(2))/min(pbar(2),pbar(4))
      addrho=0.50*gamma*abs(rhobar(4)-rhobar(2))/
     &     min(rhobar(2),rhobar(4))
      call interp(pbar,pjl,pjr,deltap,p6,flat,0,addp,addrho,dx)
      call interp(rhobar,rhojl,rhojr,deltarho,rho6,flat,1,addp,
     &     addrho,dx)
      call interp(vxbar,vxjl,vxjr,deltavx,vx6,flat,0,addp,addrho,dx)
      call interp(gbar,gjl,gjr,deltag,g6,flat,0,addp,addrho,dx)
      call interp(vybar,vyjl,vyjr,deltavy,vy6,flat,0,addp,addrho,dx)
      call interp(vzbar,vzjl,vzjr,deltavz,vz6,flat,0,addp,addrho,dx)
!     
      END SUBROUTINE boundary1
!     
!******************************************************************
!******************************************************************
!     
      SUBROUTINE boundary2(pres,ngrid,rho,vx,vy,vz,g,c,dx)

!     
!     compute boundary values of the hydro quantities with fixed
!     boundary conditions
!     

      IMPLICIT NONE
!     
!     parameters
!     
      INTEGER :: j
      INTEGER :: ngrid
      REAL*8, DIMENSION(ngrid) :: pres,rho,vx,vy,vz,g,c
      REAL*8 :: dx

!     
!     local variables
!     

      REAL*8 :: pbar(5),rhobar(5),vxbar(5),vybar(5),vzbar(5),gbar(5),
     .     cbar(2)
      REAL*8 :: rhojl(2),pjl(2),vxjl(2),vyjl(2),vzjl(2),cjl(2),gjl(2)
      REAL*8 :: rhojr(2),pjr(2),vxjr(2),vyjr(2),vzjr(2),cjr(2),gjr(2)
      REAL*8 :: deltarho(2),deltap(2),deltavx(2),deltavy(2),deltavz(2)
     .     ,deltac(2),deltag(2)
      REAL*8 :: rho6(2),p6(2),vx6(2),vy6(2),vz6(2),c6(2),g6(2)
!     
      common/bound/pbar,rhobar,vxbar,vybar,vzbar,gbar,cbar
      common/jleft/rhojl,pjl,vxjl,vyjl,vzjl,cjl,gjl
      common/jright/rhojr,pjr,vxjr,vyjr,vzjr,cjr,gjr
      common/delta/deltarho,deltap,deltavx,deltavy,deltavz,deltac,deltag
      common/a6/rho6,p6,vx6,vy6,vz6,c6,g6
!     
!     compute the interpolation functions for the virtual zone 0
!     
      do j=1,5
         pbar(j)=pres(1)
         rhobar(j)=rho(1)
         vxbar(j)=vx(1)
         vybar(j)=vy(1)
         vzbar(j)=vz(1)
         gbar(j)=g(1)
      enddo
      cbar(1)=c(1)
      cbar(2)=c(1)
!     
      call interp(pbar,pjl,pjr,deltap,p6,dx)
      call interp(rhobar,rhojl,rhojr,deltarho,rho6,dx)
      call interp(vxbar,vxjl,vxjr,deltavx,vx6,dx)
      call interp(gbar,gjl,gjr,deltag,g6,dx)
      call interp(vybar,vyjl,vyjr,deltavy,vy6,dx)
      call interp(vzbar,vzjl,vzjr,deltavz,vz6,dx)
!     
      END SUBROUTINE boundary2
      
