!     1) phi3d   I
!     2) vx3d    O
!     3) vy3d    O
!     4) vz3d    O
!     8) omega_m      I
!     10)omega_lambda I
!     11)dat     I
!     12)at      I
!     20)nx      I
!     21)ny      I
!     22)nz      I
!     23)ngrid   I

      SUBROUTINE initvel(phi3d,vx3d,vy3d,vz3d,
     &     nx,ny,nz,
     &     omega_dm,omega_bm,omega_v,at,dat)

!     
!     initial conditions for the baryonic velocity (linear approximation)
!     
!     
      IMPLICIT NONE

!     
!     parameters
!
      REAL*8, DIMENSION (nx,ny,nz) :: phi3d
      REAL*8, DIMENSION (nx,ny,nz) :: vx3d,vy3d,vz3d
      REAL*8 :: omega_dm,omega_bm,omega_v
      REAL*8 :: dat,at
      INTEGER:: nx,ny,nz

!     
!     local variables
!
      logical     ltrace
      parameter ( ltrace = .false. )


      INTEGER :: i,j,k
      INTEGER :: im1,ip1,nxny
      REAL*8 :: vvv,velocity_factor,z,omega_m
!     
      nxny=nx*ny

!     
!     calculate the correction factor to calculate velocity starting
!     from the gravitational force in linear regime
!     
      omega_m = omega_dm+omega_bm
      z=1.0/at
      velocity_factor=( (omega_m*(z)**3) / 
     &     (omega_m*(z)**3-(omega_m+omega_v-1.0)*
     &     (z)**2+omega_v) )**(4.0/7.0)
!     
      vvv=0.66666667*velocity_factor
      vvv=vvv*at/dat
      vvv=vvv/omega_m
!     
!     

      do k=1,nz
         do j=1,ny
            do i=1,nx
!     vx >
               ip1=i+1
               im1=i-1
               if(i.eq.1)im1=nx
               if(i.eq.nx)ip1=1
               vx3d(i,j,k)=vvv*(phi3d(ip1,j,k)-
     &              phi3d(im1,j,k))*0.5
!     vy >
               ip1=j+1
               im1=j-1
               if(j.eq.1)im1=ny
               if(j.eq.ny)ip1=1
               vy3d(i,j,k)=vvv*(phi3d(i,ip1,k)-
     &              phi3d(i,im1,k))*0.5


!     vz >
               ip1=k+1
               im1=k-1
               if(k.eq.1)im1=nz
               if(k.eq.nz)ip1=1
               vz3d(i,j,k)=vvv*(phi3d(i,j,ip1)-
     &              phi3d(i,j,im1))*0.5

            enddo
         enddo
      enddo


      if( ltrace ) then
         write(*,*)'INITVEL END'
         CALL checkreal(vx3d,nx,ny,nz,'VX   ')
         CALL checkreal(vy3d,nx,ny,nz,'VY   ')
         CALL checkreal(vz3d,nx,ny,nz,'VZ   ')
      endif

      END SUBROUTINE initvel
