!     

      SUBROUTINE searchsave(nes3d,p3d,vx3d,vy3d,vz3d,rhobm,
     &     pold,vxold,vyold,vzold,rhobmold,
     &     nx,ny,nz,eta1)
!     
!     search for strongly compressing regions (possible shocks)
!     save t^n values of hydro variables 


      IMPLICIT NONE

!     
!     parameters
!     

      
      INTEGER(1), DIMENSION (nx,ny,nz) :: nes3d
      REAL*8, DIMENSION (nx,ny,nz) :: p3d,vx3d,vy3d,vz3d,rhobm
      REAL*8, DIMENSION (nx,ny,nz) :: pold,vxold,vyold,vzold,rhobmold
      REAL*8 :: eta1
      INTEGER:: nx,ny,nz
!     
!     local variables
!     

!     

      logical     ltrace
      parameter ( ltrace = .false. )


      INTEGER :: indexm,indexp
      INTEGER :: i,j,k
      REAL*8 :: pgradx,vgradx,pgrady,vgrady,pgradz,vgradz,vgrad,pgrad


      if( ltrace ) then
         write(*,*)'SEARCHSAVE START'
         CALL checkreal(p3d,nx,ny,nz,'P    ')
         CALL checkreal(vx3d,nx,ny,nz,'VX   ')
         CALL checkreal(vy3d,nx,ny,nz,'VY   ')
         CALL checkreal(vz3d,nx,ny,nz,'VZ   ')
         CALL checkreal(rhobm,nx,ny,nz,'RHO  ')
         CALL checkreal(pold,nx,ny,nz,'POLD ')
         CALL checkreal(rhobmold,nx,ny,nz,'RHOOL')
         CALL checkreal(vxold,nx,ny,nz,'VXOLD')
         CALL checkreal(vyold,nx,ny,nz,'VYOLD')
         CALL checkreal(vzold,nx,ny,nz,'VZOLD')
         CALL checkbyte(nes3d,nx,ny,nz,'NES  ')
      endif

!     
!     
      pgrad=0.0
!     
      do k=1,nz
         do j=1,ny
            do i=1,nx
               nes3d(i,j,k)=IAND(nes3d(i,j,k),NOT(1))
!     
!     compute pressure and velocity gradients in the x-direction:
!     
               indexp=i+1
               indexm=i-1
               if(i.eq.nx)indexp=1
               if(i.eq.1)indexm=nx
               pgradx=abs(p3d(indexp,j,k)-p3d(indexm,j,k))
               vgradx=vx3d(indexp,j,k)-vx3d(indexm,j,k)
!     
!     y-direction:
!     
               indexp=j+1
               indexm=j-1
               if(j.eq.ny)indexp=1
               if(j.eq.1)indexm=ny
               pgrady=abs(p3d(i,indexp,k)-p3d(i,indexm,k))
               vgrady=vy3d(i,indexp,k)-vy3d(i,indexm,k)
!     
!     z-direction:
!     
               indexp=k+1
               indexm=k-1
               if(k.eq.nz)indexp=1
               if(k.eq.1)indexm=nz
               pgradz=abs(p3d(i,j,indexp)-p3d(i,j,indexm))
               vgradz=vz3d(i,j,indexp)-vz3d(i,j,indexm)
!     
               vgrad=vgradx+vgrady+vgradz
               pgrad=sqrt(pgradx*pgradx+pgrady*pgrady+pgradz*pgradz)
     &              /p3d(i,j,k)
               if(vgrad.lt.0.0.and.pgrad.gt.eta1)
     &              nes3d(i,j,k)=IOR(nes3d(i,j,k),1)

               
               rhobmold(i,j,k)=rhobm(i,j,k)
               vxold(i,j,k)=vx3d(i,j,k)
               vyold(i,j,k)=vy3d(i,j,k)
               vzold(i,j,k)=vz3d(i,j,k)
               pold(i,j,k)=p3d(i,j,k)

            enddo
         enddo
      enddo
!     
!     

      if( ltrace ) then
         write(*,*)'SEARCHSAVE END'
         CALL checkreal(pold,nx,ny,nz,'POLD ')
         CALL checkreal(rhobmold,nx,ny,nz,'RHOOL')
         CALL checkreal(vxold,nx,ny,nz,'VXOLD')
         CALL checkreal(vyold,nx,ny,nz,'VYOLD')
         CALL checkreal(vzold,nx,ny,nz,'VZOLD')
         CALL checkbyte(nes3d,nx,ny,nz,'NES  ')
      endif

!     
!     
      END SUBROUTINE searchsave
