!     Compute DM density.
!     1) rhodm3d O
!     2) x1      I 
!     3) x2      I
!     4) x3      I
!     5) amass   I
!     6) nx      I
!     7) ny      I
!     8) nz      I
!     9) nparmax I

      SUBROUTINE densitydm(rhodm,x1,x2,x3,nx,ny,nz,nparmax,amass)
!     
!     
      IMPLICIT NONE

!     
!     parameters


      REAL*8, DIMENSION (nx,ny,nz) :: rhodm
      REAL, DIMENSION (nparmax) :: x1,x2,x3
      INTEGER :: nx,ny,nz,nparmax  
      REAL*8  :: amass 

!     
!     local variables
!     

      logical     ltrace
      parameter ( ltrace = .false. )


      INTEGER :: i1,i2,i3
      INTEGER :: i11,i21,i31
      INTEGER :: j
      REAL*8 :: dd1,dd2,dd3
      REAL*8 :: de1,de2,de3
      REAL*8 :: d1,d2,d3,d4,d5,d6,d7,d8
!     
!     initialize density
!     

      rhodm=0.0


      if ( ltrace ) then
         write(*,*)'DENSITYDM START'
         CALL checkvet(x1,nparmax,'X1   ')
         CALL checkvet(x2,nparmax,'X2   ')
         CALL checkvet(x3,nparmax,'X3   ')
      endif



!     
!     distribute masses of the particles which lies in the processor
!     and belong to the processor ITSELF
!     
      do j=1,nparmax

!     
!     find position on the grid
!     
         i1=int(x1(j)+0.5)
         i2=int(x2(j)+0.5)
         i3=int(x3(j)+0.5)
!     
         dd1=x1(j)-float(i1)+0.5
         dd2=x2(j)-float(i2)+0.5
         dd3=x3(j)-float(i3)+0.5
         de1=1.0-dd1
         de2=1.0-dd2
         de3=1.0-dd3
         if(i1.eq.0) i1=nx
         if(i2.eq.0) i2=ny
         if(i3.eq.0) i3=nz
!     
!     calculate weights
!     
         d1= de1*de2*de3
         d2= dd1*de2*de3
         d3= de1*dd2*de3
         d4= dd1*dd2*de3
         d5= de1*de2*dd3
         d6= dd1*de2*dd3
         d7= de1*dd2*dd3
         d8= dd1*dd2*dd3
         i11=i1+1
         if(i11.gt.nx)i11=1
         i21=i2+1
         if(i21.gt.ny)i21=1
         i31=i3+1
         if(i31.gt.nz)i31=1
!     
!     calculate density
!     
         rhodm(i1,i2,i3)=rhodm(i1,i2,i3)+d1
         rhodm(i11,i2,i3)=rhodm(i11,i2,i3)+ d2
         rhodm(i1,i21,i3)=rhodm(i1,i21,i3)+d3
         rhodm(i11,i21,i3)=rhodm(i11,i21,i3)+d4
      
         rhodm(i1,i2,i31)=rhodm(i1,i2,i31)+d5
         rhodm(i11,i2,i31)=rhodm(i11,i2,i31)+d6
         rhodm(i1,i21,i31)=rhodm(i1,i21,i31)+d7
         rhodm(i11,i21,i31)=rhodm(i11,i21,i31)+d8

      enddo
!     
      rhodm=amass*rhodm

      if ( ltrace ) then
         write(*,*)'DENSITYDM STOP'
         CALL checkreal(rhodm,nx,ny,nz,'RHODM')
      endif


      END SUBROUTINE densitydm
