!     Compute DM density.
!     1) rhodm3d O
!     2) x1      I 
!     3) x2      I
!     4) x3      I
!     5) amass   I
!     6) nx      I
!     7) ny      I
!     8) nz      I
!     9) nparmax I

      SUBROUTINE densitydmtsc(rhodm,x1,x2,x3,nx,ny,nz,nparmax,amass)
!     Use triangular shape cloud
!     
      IMPLICIT NONE

!     
!     parameters


      REAL*8, DIMENSION (nx,ny,nz) :: rhodm
      REAL, DIMENSION (nparmax) :: x1,x2,x3
      INTEGER :: nx,ny,nz,nparmax  
      REAL*8  :: amass 

!     
!     local variables
!     

      logical     ltrace
      parameter ( ltrace = .false. )


      INTEGER :: i1,i2,i3
      INTEGER :: i11,i21,i31
      INTEGER :: i12,i22,i32
      INTEGER :: j
      REAL*8 :: dd1,dd2,dd3
      REAL*8 :: de1,de2,de3
      REAL*8 :: df1,df2,df3

      REAL*8 :: w1,w2,w3
      REAL*8 :: w11,w21,w31
      REAL*8 :: w12,w22,w32

!     
!     initialize density
!     

      rhodm=0.0


      if ( ltrace ) then
         write(*,*)'DENSITYDMTSC START'
         CALL checkvet(x1,nparmax,'X1   ')
         CALL checkvet(x2,nparmax,'X2   ')
         CALL checkvet(x3,nparmax,'X3   ')
      endif



!     
!     distribute masses of the particles which lies in the processor
!     and belong to the processor ITSELF
!     
      do j=1,nparmax

!     
!     find position on the grid
!     
         i1=int(x1(j))
         i2=int(x2(j))
         i3=int(x3(j))
!

!
         dd1=abs(x1(j)-float(i1)+0.5)
         dd2=abs(x2(j)-float(i2)+0.5)
         dd3=abs(x3(j)-float(i3)+0.5)

         de1=abs(x1(j)-float(i1+1)+0.5)
         de2=abs(x2(j)-float(i2+1)+0.5)
         de3=abs(x3(j)-float(i3+1)+0.5)

         df1=abs(x1(j)-float(i1+2)+0.5)
         df2=abs(x2(j)-float(i2+2)+0.5)
         df3=abs(x3(j)-float(i3+2)+0.5)


         if(i1.eq.0) i1=nx
         if(i2.eq.0) i2=ny
         if(i3.eq.0) i3=nz
!     
         i11=i1+1
         if(i11.gt.nx)i11=1
         i21=i2+1
         if(i21.gt.ny)i21=1
         i31=i3+1
         if(i31.gt.nz)i31=1
!
         i12=i11+1
         if(i12.gt.nx)i12=1
         i22=i21+1
         if(i22.gt.ny)i22=1
         i32=i31+1
         if(i32.gt.nz)i32=1

!     

!        

!

!     calculate weights
!     
         w1= 0.5*((1.5-dd1)**2)
         w11= 0.75 - de1**2
         w12= 0.5*((1.5-df1)**2)
!         if(w1+w11+w12.gt.1.0) then
!            write(*,*)'TSC error X:',w1+w11+w12
!            write(*,*)'x:',x1(j)
!            write(*,*)'i i1 i2:',i1,i11,i12
!            write(*,*)'dd1:',dd1
!            write(*,*)'de1:',de1
!            write(*,*)'df1:',df1
!            write(*,*)'w1 :',w1
!            write(*,*)'w11:',w11
!            write(*,*)'w12:',w12
!            write(*,*)'-----------'
!         endif
!
         w2= 0.5*((1.5-dd2)**2)
         w21= 0.75 - de2**2
         w22= 0.5*((1.5-df2)**2)
!
         w3= 0.5*((1.5-dd3)**2)
         w31= 0.75 - de3**2
         w32= 0.5*((1.5-df3)**2)      
!
!     calculate density
!     
         rhodm(i1,i2,i3)=rhodm(i1,i2,i3)+w1*w2*w3
         rhodm(i11,i2,i3)=rhodm(i11,i2,i3)+ w11*w2*w3
         rhodm(i12,i2,i3)=rhodm(i12,i2,i3)+ w12*w2*w3
         rhodm(i1,i21,i3)=rhodm(i1,i21,i3)+w1*w21*w3
         rhodm(i11,i21,i3)=rhodm(i11,i21,i3)+w11*w21*w3
         rhodm(i12,i21,i3)=rhodm(i12,i21,i3)+w12*w21*w3
         rhodm(i1,i22,i3)=rhodm(i1,i22,i3)+w1*w22*w3
         rhodm(i11,i22,i3)=rhodm(i11,i22,i3)+w11*w22*w3
         rhodm(i12,i22,i3)=rhodm(i12,i22,i3)+w12*w22*w3
!
         rhodm(i1,i2,i31)=rhodm(i1,i2,i31)+w1*w2*w31
         rhodm(i11,i2,i31)=rhodm(i11,i2,i31)+ w11*w2*w31
         rhodm(i12,i2,i31)=rhodm(i12,i2,i31)+ w12*w2*w31
         rhodm(i1,i21,i31)=rhodm(i1,i21,i31)+w1*w21*w31
         rhodm(i11,i21,i31)=rhodm(i11,i21,i31)+w11*w21*w31
         rhodm(i12,i21,i31)=rhodm(i12,i21,i31)+w12*w21*w31
         rhodm(i1,i22,i31)=rhodm(i1,i22,i31)+w1*w22*w31
         rhodm(i11,i22,i31)=rhodm(i11,i22,i31)+w11*w22*w31
         rhodm(i12,i22,i31)=rhodm(i12,i22,i31)+w12*w22*w31
!
         rhodm(i1,i2,i32)=rhodm(i1,i2,i32)+w1*w2*w32
         rhodm(i11,i2,i32)=rhodm(i11,i2,i32)+ w11*w2*w32
         rhodm(i12,i2,i32)=rhodm(i12,i2,i32)+ w12*w2*w32
         rhodm(i1,i21,i32)=rhodm(i1,i21,i32)+w1*w21*w32
         rhodm(i11,i21,i32)=rhodm(i11,i21,i32)+w11*w21*w32
         rhodm(i12,i21,i32)=rhodm(i12,i21,i32)+w12*w21*w32
         rhodm(i1,i22,i32)=rhodm(i1,i22,i32)+w1*w22*w32
         rhodm(i11,i22,i32)=rhodm(i11,i22,i32)+w11*w22*w32
         rhodm(i12,i22,i32)=rhodm(i12,i22,i32)+w12*w22*w32
      enddo
!     
      rhodm=amass*rhodm

      if ( ltrace ) then
         write(*,*)'DENSITYDMTSC STOP'
         CALL checkreal(rhodm,nx,ny,nz,'RHODM')
      endif


      END SUBROUTINE densitydmtsc
