!     1) x1      I/O
!     2) x2      I/O
!     3) x3      I/O
!     4) v1      I/0
!     5) v2      I/0
!     8) v3      I/O
!     10)gx      I 
!     11)gy      I
!     12)gz      I
!     13)gxold   I
!     14)gyold   I
!     15)gzold   I
!     16)xmax    I
!     17)dtat    I
!     18)dat     I
!     19)rdtath  I
!     19)dath    I
!     20)nx      I
!     21)ny      I
!     22)nz      I
!     23)nparmax I
!     24)dmvelmaxO

      SUBROUTINE nbody(x1,x2,x3,v1,v2,v3,
     &     gx,gy,gz,gxold,gyold,gzold,
     &     nx,ny,nz,nparmax, 
     &     at,dat,ath,dath,dt,dmvelmax)
!     

      IMPLICIT NONE

!     
!     parameters
!     

      REAL, DIMENSION (nparmax) :: x1,x2,x3
      REAL, DIMENSION (nparmax) :: v1,v2,v3
      REAL*8, DIMENSION (nx,ny,nz) :: gx,gy,gz
      REAL*8, DIMENSION (nx,ny,nz) :: gxold,gyold,gzold
      REAL*8 :: at,dat,ath,dath,dmvelmax,dt


      INTEGER :: nx,ny,nz,nparmax

!     
!     local variables
!     

      logical     ltrace
      parameter ( ltrace = .false. )

!     
      REAL*8, parameter :: pi=3.1415926, twopi=2.0*pi
      INTEGER :: j
      INTEGER :: ip1,jp1,j1,kp1,i3aux,i1aux,i2aux
      INTEGER :: iidim1
      INTEGER :: i1,i2,i3
      INTEGER :: k1
!     

      REAL*8  :: dd1,dd2,dd3,de1,de2,de3
      REAL*8  :: ffx,ffy,ffz,xmax
      REAL  :: xhalf(3),vhalf(3)
      REAL*8  :: d1,d2,d3,d4,d5,d6,d7,d8

      REAL*8 dtat,rdtath

!     
      INTEGER :: mype,npes
      mype = 0
      npes = 1


      dtat = dt*(1.0/at)
      rdtath = dt/ath

      xmax=float(nx)

      
!     
!     move particles which fall in and belong to this processor : 
!     use Lax-Wendroff second order two steps scheme
!     1)  t^n ---> t^{n+1/2}
!     2)  t^{n+1/2} ---> t^{n+1}
!     
      do j=1,nparmax
!     
         i1=int(x1(j)+0.5)
         i2=int(x2(j)+0.5)
         i3=int(x3(j)+0.5)	
         i1aux=i1
         i2aux=i2
         i3aux=i3
         if(i1.eq.0)i1=nx
         if(i2.eq.0)i2=ny
         if(i3.eq.0)i3=nz
!     
         dd1=x1(j)-float(i1aux)+0.5
         dd2=x2(j)-float(i2aux)+0.5
         dd3=x3(j)-float(i3aux)+0.5
         if(dd1.gt.1.0.or.dd1.lt.0.0)
     &        write(*,*)'WARNING: dd1 overflow'
         if(dd2.gt.1.0.or.dd2.lt.0.0)
     &        write(*,*)'WARNING: dd2 overflow'
         if(dd3.gt.1.0.or.dd3.lt.0.0)
     &        write(*,*)'WARNING: dd3 overflow'
         de1=1.0-dd1
         de2=1.0-dd2
         de3=1.0-dd3
         d1= de1*de2*de3
         d2= dd1*de2*de3
         d3= de1*dd2*de3
         d4= dd1*dd2*de3
         d5= de1*de2*dd3
         d6= dd1*de2*dd3
         d7= de1*dd2*dd3
         d8= dd1*dd2*dd3
!     
         i1=i1
         ip1=i1+1
         if(i1.eq.nx)then
            ip1=1
         endif
!     
         j1=i2
         jp1=i2+1
         if(i2.eq.ny)then
            jp1=1
         endif
!     
         k1=i3
         kp1=i3+1
         if(i3.eq.nz)then
            kp1=1
         endif
!     
         ffx=gxold(i1,j1,k1)*d1+
     &        gxold(ip1,j1,k1)*d2+
     &        gxold(i1,jp1,k1)*d3+
     &        gxold(ip1,jp1,k1)*d4+
     &        gxold(i1,j1,kp1)*d5+
     &        gxold(ip1,j1,kp1)*d6+
     &        gxold(i1,jp1,kp1)*d7+
     &        gxold(ip1,jp1,kp1)*d8
         
         ffy=gyold(i1,j1,k1)*d1+
     &        gyold(ip1,j1,k1)*d2+
     &        gyold(i1,jp1,k1)*d3+
     &        gyold(ip1,jp1,k1)*d4+
     &        gyold(i1,j1,kp1)*d5+
     &        gyold(ip1,j1,kp1)*d6+
     &        gyold(i1,jp1,kp1)*d7+
     &        gyold(ip1,jp1,kp1)*d8
         
         ffz=gzold(i1,j1,k1)*d1+
     &        gzold(ip1,j1,k1)*d2+
     &        gzold(i1,jp1,k1)*d3+
     &        gzold(ip1,jp1,k1)*d4+
     &        gzold(i1,j1,kp1)*d5+
     &        gzold(ip1,j1,kp1)*d6+
     &        gzold(i1,jp1,kp1)*d7+
     &        gzold(ip1,jp1,kp1)*d8
!     
!     update velocity and position of the particle at t^n+1/2, and set periodic 
!     boundary conditions
!     
         vhalf(1)=v1(j)*(1.0-0.5*dat*dtat)+0.5*ffx*dtat
         vhalf(2)=v2(j)*(1.0-0.5*dat*dtat)+0.5*ffy*dtat
         vhalf(3)=v3(j)*(1.0-0.5*dat*dtat)+0.5*ffz*dtat
!     
         xhalf(1)=x1(j)+0.5*v1(j)*dtat
         xhalf(2)=x2(j)+0.5*v2(j)*dtat
         xhalf(3)=x3(j)+0.5*v3(j)*dtat
         do iidim1=1,2
            if (xhalf(iidim1).lt.0.0) 
     &           xhalf(iidim1)=xhalf(iidim1)+xmax
            if (xhalf(iidim1).ge.xmax) 
     &           xhalf(iidim1)=xhalf(iidim1)-xmax
         enddo
!     

         i1=int(xhalf(1)+0.5)
         i2=int(xhalf(2)+0.5)
         i3=int(xhalf(3)+0.5)
         i1aux=i1
         i2aux=i2
         i3aux=i3
         if(i1.eq.0)i1=nx
         if(i2.eq.0)i2=ny
         if(i3.eq.0)i3=nz
!     
         dd1=xhalf(1)-float(i1aux)+0.5
         dd2=xhalf(2)-float(i2aux)+0.5
         dd3=xhalf(3)-float(i3aux)+0.5
         if(dd1.gt.1.0.or.dd1.lt.0.0)
     &        write(*,*)'WARNING: dd1 overflow'
         if(dd2.gt.1.0.or.dd2.lt.0.0)
     &        write(*,*)'WARNING: dd2 overflow'
         if(dd3.gt.1.0.or.dd3.lt.0.0)
     &        write(*,*)'WARNING: dd3 overflow'
         de1=1.0-dd1
         de2=1.0-dd2
         de3=1.0-dd3
         d1= de1*de2*de3
         d2= dd1*de2*de3
         d3= de1*dd2*de3
         d4= dd1*dd2*de3
         d5= de1*de2*dd3
         d6= dd1*de2*dd3
         d7= de1*dd2*dd3
         d8= dd1*dd2*dd3
!     
         i1=i1
         ip1=i1+1
         if(i1.eq.nx)then
            ip1=1
         endif
!     
         j1=i2
         jp1=i2+1
         if(i2.eq.ny)then
            jp1=1
         endif
!     
         k1=i3
         kp1=i3+1
         if(i3.eq.nz)then
            kp1=1
         endif
!     
         ffx=gx(i1,j1,k1)*d1+
     &        gx(ip1,j1,k1)*d2+
     &        gx(i1,jp1,k1)*d3+
     &        gx(ip1,jp1,k1)*d4+
     &        gx(i1,j1,kp1)*d5+
     &        gx(ip1,j1,kp1)*d6+
     &        gx(i1,jp1,kp1)*d7+
     &        gx(ip1,jp1,kp1)*d8
         
         ffy=gy(i1,j1,k1)*d1+
     &        gy(ip1,j1,k1)*d2+ 
     &        gy(i1,jp1,k1)*d3+
     &        gy(ip1,jp1,k1)*d4+
     &        gy(i1,j1,kp1)*d5+
     &        gy(ip1,j1,kp1)*d6+
     &        gy(i1,jp1,kp1)*d7+
     &        gy(ip1,jp1,kp1)*d8
         
         ffz=gz(i1,j1,k1)*d1+
     &        gz(ip1,j1,k1)*d2+
     &        gz(i1,jp1,k1)*d3+
     &        gz(ip1,jp1,k1)*d4+
     &        gz(i1,j1,kp1)*d5+
     &        gz(ip1,j1,kp1)*d6+
     &        gz(i1,jp1,kp1)*d7+
     &        gz(ip1,jp1,kp1)*d8
!     
!     update velocity and position of the particle at t^{n+1}, and set periodic 
!     boundary conditions
!     
         v1(j)=v1(j)-rdtath*dath*vhalf(1)+rdtath*ffx
         v2(j)=v2(j)-rdtath*dath*vhalf(2)+rdtath*ffy
         v3(j)=v3(j)-rdtath*dath*vhalf(3)+rdtath*ffz
!     
         x1(j)=x1(j)+rdtath*vhalf(1)
         x2(j)=x2(j)+rdtath*vhalf(2)
         x3(j)=x3(j)+rdtath*vhalf(3)
         if (x1(j).lt.0.0) x1(j)=x1(j)+xmax
         if (x1(j).ge.xmax) x1(j)=x1(j)-xmax
         if (x2(j).lt.0.0) x2(j)=x2(j)+xmax
         if (x2(j).ge.xmax) x2(j)=x2(j)-xmax
         if (x3(j).lt.0.0) x3(j)=x3(j)+xmax
         if (x3(j).ge.xmax) x3(j)=x3(j)-xmax
!     

!     
      enddo
      dmvelmax=maxval(v1**2+v2**2+v3**2)
      dmvelmax=sqrt(dmvelmax)



      if ( ltrace ) then
         write(*,*)'NBODY END'
         CALL checkvet(x1,nparmax,'X1   ')
         CALL checkvet(x2,nparmax,'X2   ')
         CALL checkvet(x3,nparmax,'X3   ')
         CALL checkvet(v1,nparmax,'V1   ')
         CALL checkvet(v2,nparmax,'V2   ')
         CALL checkvet(v3,nparmax,'V3   ')
         write(*,*)'dmvelmax',dmvelmax
         write(*,*)'dat = ',dat
         write(*,*)'dath = ',dath
         write(*,*)'dtat = ',dtat
         write(*,*)'rdtath = ',rdtath
      endif


!     
      END SUBROUTINE nbody
