!     1) x1      I/O
!     2) x2      I/O
!     3) x3      I/O
!     4) v1      I/0
!     5) v2      I/0
!     8) v3      I/O
!     10)gx      I 
!     11)gy      I
!     12)gz      I
!     13)gxold   I
!     14)gyold   I
!     15)gzold   I
!     16)xmax    I
!     17)dtat    I
!     18)dat     I
!     19)rdtath  I
!     19)dath    I
!     20)nx      I
!     21)ny      I
!     22)nz      I
!     23)nparmax I
!     24)dmvelmaxO

      SUBROUTINE nbodytsc(x1,x2,x3,v1,v2,v3,
     &     gx,gy,gz,gxold,gyold,gzold,
     &     nx,ny,nz,nparmax, 
     &     at,dat,ath,dath,dt,dmvelmax)
!     

      IMPLICIT NONE

!     
!     parameters
!     

      REAL*4, DIMENSION (nparmax) :: x1,x2,x3
      REAL*4, DIMENSION (nparmax) :: v1,v2,v3
      REAL*8, DIMENSION (nx,ny,nz) :: gx,gy,gz
      REAL*8, DIMENSION (nx,ny,nz) :: gxold,gyold,gzold
      REAL*8 :: at,dat,ath,dath,dmvelmax,dt


      INTEGER :: nx,ny,nz,nparmax

!     
!     local variables
!     

      logical     ltrace
      parameter ( ltrace = .false. )

!     
      REAL*8, parameter :: pi=3.1415926, twopi=2.0*pi
      INTEGER :: j
      INTEGER :: iidim1
      INTEGER :: i1,i2,i3
      INTEGER :: i11,i21,i31
      INTEGER :: i12,i22,i32

!     

      REAL*8  :: dd1,dd2,dd3,de1,de2,de3
      REAL*8  :: df1,df2,df3
      REAL*8  :: ffx,ffy,ffz,xmax
      REAL*8  :: xhalf(3),vhalf(3)


      REAL*8 :: w1,w2,w3
      REAL*8 :: w11,w21,w31
      REAL*8 :: w12,w22,w32


      REAL*8 dtat,rdtath

!     


      dtat = dt*(1.0/at)
      rdtath = dt/ath

      xmax=float(nx)     
!     
!     move particles which fall in and belong to this processor : 
!     use Lax-Wendroff second order two steps scheme
!     1)  t^n ---> t^{n+1/2}
!     2)  t^{n+1/2} ---> t^{n+1}
!     
      do j=1,nparmax
!     
         i1=int(x1(j))
         i2=int(x2(j))
         i3=int(x3(j))	
!
         dd1=abs(x1(j)-float(i1)+0.5)
         dd2=abs(x2(j)-float(i2)+0.5)
         dd3=abs(x3(j)-float(i3)+0.5)
!
         de1=abs(x1(j)-float(i1+1)+0.5)
         de2=abs(x2(j)-float(i2+1)+0.5)
         de3=abs(x3(j)-float(i3+1)+0.5)
!
         df1=abs(x1(j)-float(i1+2)+0.5)
         df2=abs(x2(j)-float(i2+2)+0.5)
         df3=abs(x3(j)-float(i3+2)+0.5)
!
         if(i1.eq.0)i1=nx
         if(i2.eq.0)i2=ny
         if(i3.eq.0)i3=nz
!     
         i11=i1+1
         if(i11.gt.nx)i11=1
         i21=i2+1
         if(i21.gt.ny)i21=1
         i31=i3+1
         if(i31.gt.nz)i31=1
!
         i12=i11+1
         if(i12.gt.nx)i12=1
         i22=i21+1
         if(i22.gt.ny)i22=1
         i32=i31+1
         if(i32.gt.nz)i32=1
!
!     calculate weights
!     
         w1= 0.5*((1.5-dd1)**2)
         w11= 0.75 - de1**2
         w12= 0.5*((1.5-df1)**2)
!
         w2= 0.5*((1.5-dd2)**2)
         w21= 0.75 - de2**2
         w22= 0.5*((1.5-df2)**2)
!
         w3= 0.5*((1.5-dd3)**2)
         w31= 0.75 - de3**2
         w32= 0.5*((1.5-df3)**2)      
!     
         ffx=gxold(i1,i2,i3)*w1*w2*w3+
     &        gxold(i11,i2,i3)* w11*w2*w3+
     &        gxold(i12,i2,i3)* w12*w2*w3+
     &        gxold(i1,i21,i3)*w1*w21*w3+
     &        gxold(i11,i21,i3)*w11*w21*w3+
     &        gxold(i12,i21,i3)*w12*w21*w3+
     &        gxold(i1,i22,i3)*w1*w22*w3+
     &        gxold(i11,i22,i3)*w11*w22*w3+
     &        gxold(i12,i22,i3)*w12*w22*w3+
     &        gxold(i1,i2,i31)*w1*w2*w31+
     &        gxold(i11,i2,i31)* w11*w2*w31+
     &        gxold(i12,i2,i31)* w12*w2*w31+
     &        gxold(i1,i21,i31)*w1*w21*w31+
     &        gxold(i11,i21,i31)*w11*w21*w31+
     &        gxold(i12,i21,i31)*w12*w21*w31+
     &        gxold(i1,i22,i31)*w1*w22*w31+
     &        gxold(i11,i22,i31)*w11*w22*w31+
     &        gxold(i12,i22,i31)*w12*w22*w31+
     &        gxold(i1,i2,i32)*w1*w2*w32+
     &        gxold(i11,i2,i32)* w11*w2*w32+
     &        gxold(i12,i2,i32)* w12*w2*w32+
     &        gxold(i1,i21,i32)*w1*w21*w32+
     &        gxold(i11,i21,i32)*w11*w21*w32+
     &        gxold(i12,i21,i32)*w12*w21*w32+
     &        gxold(i1,i22,i32)*w1*w22*w32+
     &        gxold(i11,i22,i32)*w11*w22*w32+
     &        gxold(i12,i22,i32)*w12*w22*w32         
         ffy=gyold(i1,i2,i3)*w1*w2*w3+
     &        gyold(i11,i2,i3)* w11*w2*w3+
     &        gyold(i12,i2,i3)* w12*w2*w3+
     &        gyold(i1,i21,i3)*w1*w21*w3+
     &        gyold(i11,i21,i3)*w11*w21*w3+
     &        gyold(i12,i21,i3)*w12*w21*w3+
     &        gyold(i1,i22,i3)*w1*w22*w3+
     &        gyold(i11,i22,i3)*w11*w22*w3+
     &        gyold(i12,i22,i3)*w12*w22*w3+
     &        gyold(i1,i2,i31)*w1*w2*w31+
     &        gyold(i11,i2,i31)* w11*w2*w31+
     &        gyold(i12,i2,i31)* w12*w2*w31+
     &        gyold(i1,i21,i31)*w1*w21*w31+
     &        gyold(i11,i21,i31)*w11*w21*w31+
     &        gyold(i12,i21,i31)*w12*w21*w31+
     &        gyold(i1,i22,i31)*w1*w22*w31+
     &        gyold(i11,i22,i31)*w11*w22*w31+
     &        gyold(i12,i22,i31)*w12*w22*w31+
     &        gyold(i1,i2,i32)*w1*w2*w32+
     &        gyold(i11,i2,i32)* w11*w2*w32+
     &        gyold(i12,i2,i32)* w12*w2*w32+
     &        gyold(i1,i21,i32)*w1*w21*w32+
     &        gyold(i11,i21,i32)*w11*w21*w32+
     &        gyold(i12,i21,i32)*w12*w21*w32+
     &        gyold(i1,i22,i32)*w1*w22*w32+
     &        gyold(i11,i22,i32)*w11*w22*w32+
     &        gyold(i12,i22,i32)*w12*w22*w32
         ffz=gzold(i1,i2,i3)*w1*w2*w3+
     &        gzold(i11,i2,i3)* w11*w2*w3+
     &        gzold(i12,i2,i3)* w12*w2*w3+
     &        gzold(i1,i21,i3)*w1*w21*w3+
     &        gzold(i11,i21,i3)*w11*w21*w3+
     &        gzold(i12,i21,i3)*w12*w21*w3+
     &        gzold(i1,i22,i3)*w1*w22*w3+
     &        gzold(i11,i22,i3)*w11*w22*w3+
     &        gzold(i12,i22,i3)*w12*w22*w3+
     &        gzold(i1,i2,i31)*w1*w2*w31+
     &        gzold(i11,i2,i31)* w11*w2*w31+
     &        gzold(i12,i2,i31)* w12*w2*w31+
     &        gzold(i1,i21,i31)*w1*w21*w31+
     &        gzold(i11,i21,i31)*w11*w21*w31+
     &        gzold(i12,i21,i31)*w12*w21*w31+
     &        gzold(i1,i22,i31)*w1*w22*w31+
     &        gzold(i11,i22,i31)*w11*w22*w31+
     &        gzold(i12,i22,i31)*w12*w22*w31+
     &        gzold(i1,i2,i32)*w1*w2*w32+
     &        gzold(i11,i2,i32)* w11*w2*w32+
     &        gzold(i12,i2,i32)* w12*w2*w32+
     &        gzold(i1,i21,i32)*w1*w21*w32+
     &        gzold(i11,i21,i32)*w11*w21*w32+
     &        gzold(i12,i21,i32)*w12*w21*w32+
     &        gzold(i1,i22,i32)*w1*w22*w32+
     &        gzold(i11,i22,i32)*w11*w22*w32+
     &        gzold(i12,i22,i32)*w12*w22*w32
!     
!     update velocity and position of the particle at t^n+1/2, and set periodic 
!     boundary conditions
!     
         vhalf(1)=v1(j)*(1.0-0.5*dat*dtat)+0.5*ffx*dtat
         vhalf(2)=v2(j)*(1.0-0.5*dat*dtat)+0.5*ffy*dtat
         vhalf(3)=v3(j)*(1.0-0.5*dat*dtat)+0.5*ffz*dtat
!     
         xhalf(1)=x1(j)+0.5*v1(j)*dtat
         xhalf(2)=x2(j)+0.5*v2(j)*dtat
         xhalf(3)=x3(j)+0.5*v3(j)*dtat
         do iidim1=1,2
            if (xhalf(iidim1).lt.0.0) 
     &           xhalf(iidim1)=xhalf(iidim1)+xmax
            if (xhalf(iidim1).ge.xmax) 
     &           xhalf(iidim1)=xhalf(iidim1)-xmax
         enddo
!     

         i1=int(xhalf(1))
         i2=int(xhalf(2))
         i3=int(xhalf(3))
!
         dd1=abs(xhalf(1)-float(i1)+0.5)
         dd2=abs(xhalf(2)-float(i2)+0.5)
         dd3=abs(xhalf(3)-float(i3)+0.5)
!
         de1=abs(xhalf(1)-float(i1+1)+0.5)
         de2=abs(xhalf(2)-float(i2+1)+0.5)
         de3=abs(xhalf(3)-float(i3+1)+0.5)
!
         df1=abs(xhalf(1)-float(i1+2)+0.5)
         df2=abs(xhalf(2)-float(i2+2)+0.5)
         df3=abs(xhalf(3)-float(i3+2)+0.5)
!
         if(i1.eq.0)i1=nx
         if(i2.eq.0)i2=ny
         if(i3.eq.0)i3=nz
!     
         i11=i1+1
         if(i11.gt.nx)i11=1
         i21=i2+1
         if(i21.gt.ny)i21=1
         i31=i3+1
         if(i31.gt.nz)i31=1
!
         i12=i11+1
         if(i12.gt.nx)i12=1
         i22=i21+1
         if(i22.gt.ny)i22=1
         i32=i31+1
         if(i32.gt.nz)i32=1
!
!     calculate weights
!     
         w1= 0.5*((1.5-dd1)**2)
         w11= 0.75 - de1**2
         w12= 0.5*((1.5-df1)**2)
!
         w2= 0.5*((1.5-dd2)**2)
         w21= 0.75 - de2**2
         w22= 0.5*((1.5-df2)**2)
!
         w3= 0.5*((1.5-dd3)**2)
         w31= 0.75 - de3**2
         w32= 0.5*((1.5-df3)**2)      
!     
         ffx=gx(i1,i2,i3)*w1*w2*w3+
     &        gx(i11,i2,i3)* w11*w2*w3+
     &        gx(i12,i2,i3)* w12*w2*w3+
     &        gx(i1,i21,i3)*w1*w21*w3+
     &        gx(i11,i21,i3)*w11*w21*w3+
     &        gx(i12,i21,i3)*w12*w21*w3+
     &        gx(i1,i22,i3)*w1*w22*w3+
     &        gx(i11,i22,i3)*w11*w22*w3+
     &        gx(i12,i22,i3)*w12*w22*w3+
     &        gx(i1,i2,i31)*w1*w2*w31+
     &        gx(i11,i2,i31)* w11*w2*w31+
     &        gx(i12,i2,i31)* w12*w2*w31+
     &        gx(i1,i21,i31)*w1*w21*w31+
     &        gx(i11,i21,i31)*w11*w21*w31+
     &        gx(i12,i21,i31)*w12*w21*w31+
     &        gx(i1,i22,i31)*w1*w22*w31+
     &        gx(i11,i22,i31)*w11*w22*w31+
     &        gx(i12,i22,i31)*w12*w22*w31+
     &        gx(i1,i2,i32)*w1*w2*w32+
     &        gx(i11,i2,i32)* w11*w2*w32+
     &        gx(i12,i2,i32)* w12*w2*w32+
     &        gx(i1,i21,i32)*w1*w21*w32+
     &        gx(i11,i21,i32)*w11*w21*w32+
     &        gx(i12,i21,i32)*w12*w21*w32+
     &        gx(i1,i22,i32)*w1*w22*w32+
     &        gx(i11,i22,i32)*w11*w22*w32+
     &        gx(i12,i22,i32)*w12*w22*w32         
         ffy=gy(i1,i2,i3)*w1*w2*w3+
     &        gy(i11,i2,i3)* w11*w2*w3+
     &        gy(i12,i2,i3)* w12*w2*w3+
     &        gy(i1,i21,i3)*w1*w21*w3+
     &        gy(i11,i21,i3)*w11*w21*w3+
     &        gy(i12,i21,i3)*w12*w21*w3+
     &        gy(i1,i22,i3)*w1*w22*w3+
     &        gy(i11,i22,i3)*w11*w22*w3+
     &        gy(i12,i22,i3)*w12*w22*w3+
     &        gy(i1,i2,i31)*w1*w2*w31+
     &        gy(i11,i2,i31)* w11*w2*w31+
     &        gy(i12,i2,i31)* w12*w2*w31+
     &        gy(i1,i21,i31)*w1*w21*w31+
     &        gy(i11,i21,i31)*w11*w21*w31+
     &        gy(i12,i21,i31)*w12*w21*w31+
     &        gy(i1,i22,i31)*w1*w22*w31+
     &        gy(i11,i22,i31)*w11*w22*w31+
     &        gy(i12,i22,i31)*w12*w22*w31+
     &        gy(i1,i2,i32)*w1*w2*w32+
     &        gy(i11,i2,i32)* w11*w2*w32+
     &        gy(i12,i2,i32)* w12*w2*w32+
     &        gy(i1,i21,i32)*w1*w21*w32+
     &        gy(i11,i21,i32)*w11*w21*w32+
     &        gy(i12,i21,i32)*w12*w21*w32+
     &        gy(i1,i22,i32)*w1*w22*w32+
     &        gy(i11,i22,i32)*w11*w22*w32+
     &        gy(i12,i22,i32)*w12*w22*w32
         ffz=gz(i1,i2,i3)*w1*w2*w3+
     &        gz(i11,i2,i3)* w11*w2*w3+
     &        gz(i12,i2,i3)* w12*w2*w3+
     &        gz(i1,i21,i3)*w1*w21*w3+
     &        gz(i11,i21,i3)*w11*w21*w3+
     &        gz(i12,i21,i3)*w12*w21*w3+
     &        gz(i1,i22,i3)*w1*w22*w3+
     &        gz(i11,i22,i3)*w11*w22*w3+
     &        gz(i12,i22,i3)*w12*w22*w3+
     &        gz(i1,i2,i31)*w1*w2*w31+
     &        gz(i11,i2,i31)* w11*w2*w31+
     &        gz(i12,i2,i31)* w12*w2*w31+
     &        gz(i1,i21,i31)*w1*w21*w31+
     &        gz(i11,i21,i31)*w11*w21*w31+
     &        gz(i12,i21,i31)*w12*w21*w31+
     &        gz(i1,i22,i31)*w1*w22*w31+
     &        gz(i11,i22,i31)*w11*w22*w31+
     &        gz(i12,i22,i31)*w12*w22*w31+
     &        gz(i1,i2,i32)*w1*w2*w32+
     &        gz(i11,i2,i32)* w11*w2*w32+
     &        gz(i12,i2,i32)* w12*w2*w32+
     &        gz(i1,i21,i32)*w1*w21*w32+
     &        gz(i11,i21,i32)*w11*w21*w32+
     &        gz(i12,i21,i32)*w12*w21*w32+
     &        gz(i1,i22,i32)*w1*w22*w32+
     &        gz(i11,i22,i32)*w11*w22*w32+
     &        gz(i12,i22,i32)*w12*w22*w32
!     
!     
!     update velocity and position of the particle at t^{n+1}, and set periodic 
!     boundary conditions
!     
         v1(j)=v1(j)-rdtath*dath*vhalf(1)+rdtath*ffx
         v2(j)=v2(j)-rdtath*dath*vhalf(2)+rdtath*ffy
         v3(j)=v3(j)-rdtath*dath*vhalf(3)+rdtath*ffz
!     
         x1(j)=x1(j)+rdtath*vhalf(1)
         x2(j)=x2(j)+rdtath*vhalf(2)
         x3(j)=x3(j)+rdtath*vhalf(3)
         if (x1(j).lt.0.0) x1(j)=x1(j)+xmax
         if (x1(j).ge.xmax) x1(j)=x1(j)-xmax
         if (x2(j).lt.0.0) x2(j)=x2(j)+xmax
         if (x2(j).ge.xmax) x2(j)=x2(j)-xmax
         if (x3(j).lt.0.0) x3(j)=x3(j)+xmax
         if (x3(j).ge.xmax) x3(j)=x3(j)-xmax
!     

!     
      enddo
      dmvelmax=maxval(v1**2+v2**2+v3**2)
      dmvelmax=sqrt(dmvelmax)



      if ( ltrace ) then
         write(*,*)'NBODYTSC END'
         CALL checkvet(x1,nparmax,'X1   ')
         CALL checkvet(x2,nparmax,'X2   ')
         CALL checkvet(x3,nparmax,'X3   ')
         CALL checkvet(v1,nparmax,'V1   ')
         CALL checkvet(v2,nparmax,'V2   ')
         CALL checkvet(v3,nparmax,'V3   ')
         write(*,*)'dmvelmax',dmvelmax
         write(*,*)'dat = ',dat
         write(*,*)'dath = ',dath
         write(*,*)'dtat = ',dtat
         write(*,*)'rdtath = ',rdtath
      endif


!     
      END SUBROUTINE nbodytsc
