#include "fftw.h"
#include "config.h"

static fftw_plan planfw[3];
static fftw_plan planbw[3];

#ifdef __cplusplus
          extern "C" { /* prevent C++ name mangling */
#endif

#define FCCREATE_PLAN FC_FUNC_(create_plan,CREATE_PLAN)
#define FCFFT_XY FC_FUNC_(fft_xy,FFT_XY)
#define FCFFT_Z FC_FUNC_(fft_z,FFT_Z)

int FCCREATE_PLAN(int *nx, int *ny, int *nz)
{
  planfw[0] = fftw_create_plan(*nx,FFTW_FORWARD,FFTW_ESTIMATE | FFTW_IN_PLACE);
  planfw[1] = fftw_create_plan(*ny,FFTW_FORWARD,FFTW_ESTIMATE | FFTW_IN_PLACE);
  planfw[2] = fftw_create_plan(*nz,FFTW_FORWARD,FFTW_ESTIMATE | FFTW_IN_PLACE);
  
  planbw[0] =fftw_create_plan(*nx,FFTW_BACKWARD,FFTW_ESTIMATE | FFTW_IN_PLACE);
  planbw[1] =fftw_create_plan(*ny,FFTW_BACKWARD,FFTW_ESTIMATE | FFTW_IN_PLACE);
  planbw[2] =fftw_create_plan(*nz,FFTW_BACKWARD,FFTW_ESTIMATE | FFTW_IN_PLACE);
  return 0;
}


int FCFFT_XY(int *isign, FFTW_COMPLEX *a, int *nx, int *ny, int *nz, 
           int *ldx, int *ldy )
{
  
  int i;
  int xstride, ystride, bigstride;
  int xhowmany, yhowmany, xidist, yidist;
  
  /* trasform  along x and y */
  bigstride = (*ldx) * (*ldy);
  
  xhowmany = (*ny);
  xstride  = 1;
  xidist   = (*ldx);
  
  yhowmany = (*nx);
  ystride  = (*ldx);
  yidist   = 1;
  
  if(*isign == 1) {
    for(i = 0; i < *nz ; i++) {
      /* trasform  along x */
      fftw(planbw[0],xhowmany,&a[i*bigstride],xstride,xidist,0,0,0);
      /* trasform  along y */
      fftw(planbw[1],yhowmany,&a[i*bigstride],ystride,yidist,0,0,0);
    }
  } else {
    for(i = 0; i < *nz ; i++) {
      /* trasform  along x */
      fftw(planfw[0],xhowmany,&a[i*bigstride],xstride,xidist,0,0,0);
      /* trasform  along y */
      fftw(planfw[1],yhowmany,&a[i*bigstride],ystride,yidist,0,0,0);
    }
  }
  return 0;
}


int FCFFT_Z(int *isign, FFTW_COMPLEX *a, int *nx, int *ny, int *nz, 
	     int *ldx, int *ldy)
{
  
  int i,stride, bigstride;
  int howmany, idist;
  
  bigstride = (*ldx) * (*ldy);
  howmany = (*ny) ;
  stride  = 1; 
  idist   = (*ldx);
  if(*isign == 1) {
    for(i=0;i<*nz;i++) {
      fftw(planbw[2],howmany,&a[i*bigstride],1,idist,0,0,0);
    }
  } else {
    for(i=0;i<*nz;i++) {
      fftw(planfw[2],howmany,&a[i*bigstride],1,idist,0,0,0);
    }
  }
  
  return 0;
}


#ifdef __cplusplus
      } /* prevent C++ name mangling */
#endif
