
!     1) A       I
!     2) iphig   I
!     3) gmat    O
!     7) grav_shap I
!     8) ga      I
!     10)ak      I
!     11)akq     I -> akq is the square of ak
!     12)nx      I
!     13)ny      I
!     14)nz      I
!     15)n11     I
!     16)n11pe   I
!     17)desca   I
!     18)n1pe    I

      SUBROUTINE gravforce(A,
     &     grav_shap,gmat,ak,
     &     akq,nx,ny,nz,ga,iphig)
!     
      IMPLICIT NONE
!     
!     parameters
!

      REAL*8, DIMENSION (nx,ny,nz) :: gmat
      REAL*8, DIMENSION (0:nx-1,0:ny-1,0:nz-1) :: grav_shap
      REAL*8 :: ga
      REAL*8, DIMENSION(0:nx-1) :: ak,akq
      INTEGER :: nx,ny,nz,iphig
      COMPLEX*16 :: A(nx,ny,nz)
!     
!     local variables
!     

      logical     ltrace
      parameter ( ltrace = .false. )

!     
      INTEGER :: i,j,k,error,desca(12)
      INTEGER :: ip1,jp1,kp1
      REAL*8  :: akk,ainvakk

      COMPLEX*16, DIMENSION (:,:,:), ALLOCATABLE :: B
!     

      desca=0
      desca(1)=nx
      desca(2)=ny
      desca(3)=nz

      desca(5)=0
      desca(6)=1

      desca(8)=nx
      desca(9)=ny


      allocate(B(nx,ny,nz),STAT=error)
      if(error.ne.0)then
         write(*,*)' failed in allocating array B in GRAVFIELDS'
         stop
      endif
   
!     
!     calculate gravitational forces >
!     
      B(1,1,1) = (0.0, 0.0)

      if(iphig.eq.1)then
         B(1,1,1) = (0.0, 0.0)
         
         do k = 0, nz-1
            kp1=k+1
            do j = 0, ny-1
               jp1=j+1
               do i = 1, nx-1
                  ip1=i+1
                  akk = akq(i) + akq(j) + akq(k)
                  ainvakk = 1. / akk
                  B(ip1,jp1,kp1) = ak(i) * dcmplx(0.0,1.0) *
     &                 A(ip1,jp1,kp1) *
     &                 ga * grav_shap(i,j,k) * ainvakk
              enddo
            enddo
          enddo

          do k = 0, nz-1
             kp1=k+1
             do j = 0, ny-1
                jp1=j+1
                i = 0
                ip1=i+1
                akk = akq(i) + akq(j) + akq(k)
                if (akk .ne. 0.0) then
                   ainvakk = 1. / akk
                   B(ip1,jp1,kp1) = ak(i) * dcmplx(0.0,1.0) *
     &                  A(ip1,jp1,kp1) *
     &                  ga * grav_shap(i,j,k) * ainvakk
              endif
            enddo
          enddo

       else if(iphig.eq.2)then
          B(1,1,1) = (0.0, 0.0)
          do k = 0, nz-1
             kp1=k+1
             do j = 0, ny-1
                jp1=j+1
                do i = 1, nx-1
                   ip1=i+1
                   akk = akq(i) + akq(j) + akq(k)
                   ainvakk = 1. / akk
                   B(ip1,jp1,kp1) = ak(j) * dcmplx(0.0,1.0) *
     &                  A(ip1,jp1,kp1) *
     &                  ga * grav_shap(i,j,k) * ainvakk
              enddo
            enddo
         enddo
!     
         do k = 0, nz-1
            kp1=k+1
            do j = 0, ny-1
               jp1=j+1
               i = 0
               ip1=i+1
               akk = akq(i) + akq(j) + akq(k)
               if (akk .ne. 0.0) then
                  ainvakk = 1. / akk
                  B(ip1,jp1,kp1) = ak(j) * dcmplx(0.0,1.0) * 
     &                 A(ip1,jp1,kp1) *
     &                 ga * grav_shap(i,j,k) * ainvakk
              endif
            enddo
          enddo

       else if(iphig.eq.3)then
          B(1,1,1) = (0.0, 0.0)
          do k = 0, nz-1
             kp1=k+1
             do j = 0, ny-1
                jp1=j+1
                do i = 1, nx-1
                   ip1=i+1
                   akk = akq(i) + akq(j) + akq(k)
                   ainvakk = 1. / akk
                   B(ip1,jp1,kp1) = ak(k) * dcmplx(0.0,1.0) *
     &                  A(ip1,jp1,kp1) *
     &                  ga * grav_shap(i,j,k) * ainvakk
              enddo
           enddo
        enddo
!     
        do k = 0, nz-1
           kp1=k+1
           do j = 0, ny-1
              jp1=j+1
              i = 0
              ip1=i+1
              akk = akq(i) + akq(j) + akq(k)
              if (akk .ne. 0.0) then
                 ainvakk = 1. / akk
                 B(ip1,jp1,kp1) = ak(k) * dcmplx(0.0,1.0) *
     &                A(ip1,jp1,kp1) *
     &                ga * grav_shap(i,j,k) * ainvakk
              endif
           enddo
        enddo

      endif
      call pc3fft(B,desca,-1)
!     
      do k=1,nz
         do j=1,ny
            do i=1,nx
               gmat(i,j,k)=-real(B(i,j,k))
            enddo
         enddo
      enddo 
      deallocate(B)


      if( ltrace ) then
         CALL checkreal(gmat,nx,ny,nz,'GMAT ')
      endif

!     
      END SUBROUTINE gravforce


