!     1) phi3d   I/O
!     2) phiold3d  I/O
!     2) gx      I/O
!     3) gy      I/O
!     4) gz      I/O
!     5) gxold   I/O
!     8) gyold   I/O
!     10)gzold   I/O
!     11)nstep   I
!     12)dt      I
!     13)dtold   I
!     14)nx      I
!     15)ny      I
!     16)nz      I



      SUBROUTINE gravhalf(phi3d,phiold3d,gx,gy,gz,
     &     gxold,gyold,gzold,nx,ny,nz,dt,dtold,nsteps)
!     
      IMPLICIT NONE
!     
!     parameters
!     

      REAL*8, DIMENSION (nx,ny,nz) :: phi3d,phiold3d
      REAL*8, DIMENSION (nx,ny,nz) :: gx,gy,gz
      REAL*8, DIMENSION (nx,ny,nz) :: gxold,gyold,gzold
      INTEGER :: nsteps      
      REAL*8  :: dt,dtold
      INTEGER :: nx,ny,nz


!     
!     local variables
!     
      logical     ltrace
      parameter ( ltrace = .false. )

      INTEGER :: i,j,k
      REAL*8 :: dephi,phih,rdtold

!
! communicate ghost planes
!

!      gx(:,:,nz+1)=gx(:,:,1)
!      gx(:,:,0)   =gx(:,:,nz)
!      gy(:,:,nz+1)=gy(:,:,1)
!      gy(:,:,0)   =gy(:,:,nz)
!      gz(:,:,nz+1)=gz(:,:,1)
!      gz(:,:,0)   =gz(:,:,nz)

!     
!     extrapolate forces at half time step
!     

      if(nsteps.eq.0)then
         gxold(:,:,:)=gx(:,:,:)
         gyold(:,:,:)=gy(:,:,:)
         gzold(:,:,:)=gz(:,:,:)
         phiold3d(:,:,:)=phi3d(:,:,:)
      endif

!     
      rdtold=1./dtold
!     
      do k=1,nz
         do j=1,ny
            do i=1,nx
               dephi=(phi3d(i,j,k)-phiold3d(i,j,k))*rdtold
               phih=dephi*0.5*dt+phi3d(i,j,k)
               phiold3d(i,j,k)=phi3d(i,j,k)
               phi3d(i,j,k)=phih
            enddo
         enddo
      enddo
!     

      do k=1,nz
         do j=1,ny
            do i=1,nx
!     
               dephi=(gx(i,j,k)-gxold(i,j,k))*rdtold
               phih=dephi*0.5*dt+gx(i,j,k)
               gxold(i,j,k)=gx(i,j,k)
               gx(i,j,k)=phih
!     
               dephi=(gy(i,j,k)-gyold(i,j,k))*rdtold
               phih=dephi*0.5*dt+gy(i,j,k)
               gyold(i,j,k)=gy(i,j,k)
               gy(i,j,k)=phih
!     
               dephi=(gz(i,j,k)-gzold(i,j,k))*rdtold
               phih=dephi*0.5*dt+gz(i,j,k)
               gzold(i,j,k)=gz(i,j,k)
               gz(i,j,k)=phih
            enddo
         enddo
      enddo

      if( ltrace ) then
         write(*,*)'GRAVHALF END'
         CALL checkreal(phi3d,nx,ny,nz,'PHI  ')
         CALL checkreal(phiold3d,nx,ny,nz,'PHIOL')
         CALL checkreal(gx,nx,ny,nz,'GX   ')
         CALL checkreal(gy,nx,ny,nz,'GY   ')
         CALL checkreal(gz,nx,ny,nz,'GZ   ')
         CALL checkreal(gxold,nx,ny,nz,'GXOLD')
         CALL checkreal(gyold,nx,ny,nz,'GYOLD')
         CALL checkreal(gzold,nx,ny,nz,'GZOLD')
      endif


!     
      END SUBROUTINE gravhalf
