!     1) A       O
!     6) phi3d   O
!        rhodm3d I
!        rhobm3d I
!     7) grav_shap I
!     8) ga      I
!     10)ak      O
!     10)akq     O
!     12)nx      I
!     13)ny      I
!     14)nz      I
!     15)n11     I
!     16)n11pe   I
!     17)desca   I
!     18)n1pe    I


      SUBROUTINE gravpot(A,phi3d,rhodm3d,rhobm3d,
     .     grav_shap,ak,akq,nx,ny,nz,ga)
!     


      IMPLICIT NONE

!    
!     parameters
!
      COMPLEX*16 :: A(nx,ny,nz)
      REAL*8, DIMENSION (nx,ny,nz) :: phi3d
      REAL*8, DIMENSION (nx,ny,nz) :: rhobm3d
      REAL*8, DIMENSION (nx,ny,nz) :: rhodm3d
      REAL*8, DIMENSION (0:nx-1,0:ny-1,0:nz-1) :: grav_shap
      REAL*8 :: ga
      REAL*8, DIMENSION(0:nx-1) :: ak,akq
      INTEGER ::nx,ny,nz
!     
!     local variables
!     
!
!     
      logical     ltrace
      parameter ( ltrace = .false. )


      INTEGER :: i,j,k,error
      INTEGER :: ip1,jp1,kp1,desca(12)
      REAL*8  :: akk,ainvakk
      COMPLEX*16, DIMENSION (:,:,:), ALLOCATABLE :: B


!     initialize k space

      CALL start(ak,akq,nx)



      phi3d=rhodm3d+rhobm3d      

      desca=0
      desca(1)=nx
      desca(2)=ny
      desca(3)=nz

      desca(5)=0
      desca(6)=1

      desca(8)=nx
      desca(9)=ny

      allocate(B(nx,ny,nz),STAT=error)

      if(error.ne.0)then
         write(*,*)' failed in allocating array B in GRAVFIELDS0'
         stop
      endif
!     

      A=dcmplx(phi3d,0.0D0)

      if ( ltrace ) then
         CALL checkcomp(A,nx,ny,nz,'A pre')
      endif

      call pc3fft(A,desca,1)

      if ( ltrace ) then
         CALL checkcomp(A,nx,ny,nz,'A pos')
      endif

      A(1,1,1) = (0.0, 0.0)
      B(1,1,1) = (0.0, 0.0)         
      
!     calculate gravitational field >
!     

      do k = 0, nz-1
         kp1=k+1
         do j = 0, ny-1
            jp1=j+1
            do i = 1, nx-1
               ip1=i+1
               akk = akq(i) + akq(j) + akq(k)
               ainvakk = 1. / akk
               B(ip1,jp1,kp1) = A(ip1,jp1,kp1) * ga *
     &              grav_shap(i,j,k) * ainvakk
            enddo
         enddo
      enddo

!     
!     fa il piano x=0
!     
      do k = 0, nz-1
         kp1=k+1
         do j = 0, ny-1
            jp1=j+1
            i = 0
            ip1=i+1
            akk = akq(i) + akq(j) + akq(k)
            if (akk .ne. 0.0) then
               ainvakk = 1. / akk
               B(ip1,jp1,kp1) = A(ip1,jp1,kp1) * ga *
     &              grav_shap(i,j,k) * ainvakk
            endif
         enddo
      enddo


!     
!     FFT transform from k to real-space
!     
      if ( ltrace ) then
         CALL checkcomp(B,nx,ny,nz,'B pre')
      endif

      call pc3fft(B,desca,-1)

      if ( ltrace ) then
         CALL checkcomp(B,nx,ny,nz,'B pos')
      endif
!     
      phi3d=real(B)
!     
      deallocate(B)


      if( ltrace ) then
         CALL checkreal(phi3d,nx,ny,nz,'PHI  ')
      endif

      END SUBROUTINE gravpot
