#ifndef _BARYONIC_H_
#define _BARYONIC_H_ 1

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* 
   Include the baryonic matter methods
 */


/* *********** Methods ********** */

/* Baryonic matter initialisation */

#define initbm  FC_FUNC(initbm,INITBM)
#ifdef __cplusplus
extern "C" {
#endif

  extern void initbm (double *, double *,double *,
		      int *, int *, int *,
		      double *,double*,double*,double *);

#ifdef __cplusplus
}
#endif

/* Initial conditions for the baryonic velocity (linear approximation) */

#define initvel  FC_FUNC(initvel,INITVEL)
#ifdef __cplusplus
extern "C" {
#endif

  extern void initvel (double *, double *,double *,double *,
		      int *, int *, int *,
		      double *,double*,double*,double *,double *);

#ifdef __cplusplus
}
#endif

/* Search for strongly compressing regions (possible shocks)
   save t^n values of hydro variables */


#define searchsave  FC_FUNC(searchsave,SEARCHSAVE)
#ifdef __cplusplus
extern "C" {
#endif

  extern void searchsave (unsigned char *, double *,double *,double *,double *,double *,
			  double *,double *,double *,double *,double *,
			  int *, int *, int *,double *);

#ifdef __cplusplus
}
#endif


/* Integrate hydro equation in the x direction */


#define lx  FC_FUNC(lx,LX)
#ifdef __cplusplus
extern "C" {
#endif

  extern void lx (double *,double *,double *,double *,double *,
		  double *,double *,double *,double *,unsigned char *,
		  int *, int *, int *,
		  double *,double *,double *,double *,double *,
		  double *,double *,double *,double *);

#ifdef __cplusplus
}
#endif

/* Integrate hydro equation in the y direction */


#define ly  FC_FUNC(ly,LY)
#ifdef __cplusplus
extern "C" {
#endif

  extern void ly (double *,double *,double *,double *,double *,
		  double *,double *,double *,double *,unsigned char *,
		  int *, int *, int *,
		  double *,double *,double *,double *,double *,
		  double *,double *,double *,double *);

#ifdef __cplusplus
}
#endif

/* Integrate hydro equation in the z direction */


#define lz  FC_FUNC(lz,LZ)
#ifdef __cplusplus
extern "C" {
#endif

  extern void lz (double *,double *,double *,double *,double *,
		  double *,double *,double *,double *,unsigned char *,
		  int *, int *, int *,
		  double *,double *,double *,double *,double *,
		  double *,double *,double *,double *);

#ifdef __cplusplus
}
#endif

/* Integrate hydro equation in the x or y or z direction */


#define lxlylz  FC_FUNC(lxlylz,LXLYLZ)
#ifdef __cplusplus
extern "C" {
#endif

  extern void lxlylz (double *,double *,double *,double *,double *,
		      double *,double *,double *,double *,unsigned char *,
		      int *, int *, int *,
		      double *,double *,double *,double *,double *,
		      double *,double *,double *,double *, int *);

#ifdef __cplusplus
}
#endif


/* Compute the contribution of the expansion on the hydro variables
   calculate maximum velocity */


#define expandspeed  FC_FUNC(expandspeed,EXPANDSPEED)
#ifdef __cplusplus
extern "C" {
#endif

  extern void expandspeed (double *,double *,double *,double *,double *,
			   double *,double *,double *,double *,
			   int *, int *, int *,
			   double *,double *,double *,double *,double *);

#ifdef __cplusplus
}
#endif

/*Calculate the baryonic matter component */


#define barmatter  FC_FUNC(barmatter,BARMATTER)
#ifdef __cplusplus
extern "C" {
#endif

  extern void barmatter(unsigned char *, double *,double *,double *,double *,
			double *,double *,double *,
			int *, int *, int *,
			double *,double *,double *,double *,double *,
			double *,double *,double *,double *,double *,
			double *,int *,int *,double *);

#ifdef __cplusplus
}
#endif



/* Divide */


#define ldivide  FC_FUNC(ldivide,LDIVIDE)
#ifdef __cplusplus
extern "C" {
#endif

  extern void ldivide (double *,double *,double *,double *,double *,
			double *,double *,double *,double *,unsigned char *,
			int *, int *, int *,
			double *,double *,double *,double *,double *,
			double *,double *,double *,double *,unsigned char *,
			int *, int *,int *,int *, int *, int *);

#ifdef __cplusplus
}
#endif

/* Divide */


#define lmerge  FC_FUNC(lmerge,LMERGE)
#ifdef __cplusplus
extern "C" {
#endif

  extern void lmerge (double *,double *,double *,double *,double *,unsigned char *,
		      int *, int *, int *,
		      double *,double *,double *,double *,double *,unsigned char *,
		      int *, int *,int *,int *, int *,int *);

#ifdef __cplusplus
}
#endif

#endif /* _BARYONIC_H */
