#include <stdio.h>
#include <stdlib.h>
#include <error.h>
#include <unistd.h>
#include <wait.h>

#include "init.h"

#define NBUF 100

void usegrafic(float *x1,float *x2,float *x3, float *v1,float *v2,float *v3,
	       int *nx, int *ny, int *nz, int *np, double *t0h0, double *at, double *vfact,
	       double *omega_dm, double *omega_bm, double *omega_v, double *hnow, 
	       double *spectidx, double *sigma_8, char *rseed)
{
  
  char buf[NBUF];
  char oldwd[NBUF];
  char dir[21]="/tmp/usegraficXXXXXX";
  char *tmpdir;
  int nparmax;
  FILE *finput;
  int ret,step,error;


  /* **************** Initialization time ******************/

  do{
    printf("(0).");
    error = 0;
    step = 1;			/* Creating temporary directory */
    tmpdir=mkdtemp(dir);  
    if(tmpdir==NULL) {error=1; break;}
    
    printf("(1).");
    step = 2; 			/* Momorize current working directory */
    if(getcwd(oldwd,NBUF)==NULL) {error=2; break;}

    printf("(2).");
    step = 3;			/* Change working directory */
    if(chdir(tmpdir)==-1) {error=3; break;}

    printf("(3).") ;   
    step = 4;			/* Create the program to executable  from number of particles  */
    if(snprintf(buf,NBUF,"%s < initial.in >/dev/null","grafic")<0) {error=4; break;}
    
    printf("(4).");
    step = 5;		        /* Writing initial.in, input file for grafic */
    finput=fopen("initial.in","w+");
    if(!finput) {error=5; break;}

    /* Select grid size */
    fprintf(finput,"%d\n",*np);	
    /* Select type of initial power spectrum */
    fprintf(finput,"2\n");	
    /* Enter Omega_m, Omega_v, H0*/
    fprintf(finput,"%f %f %f\n",(*omega_dm)+(*omega_bm),*omega_v,(*hnow)*100);		
    /* Enter Omega_BM, */
    fprintf(finput,"%f\n",(*omega_bm));	
    /* Enter long-wave spectral index n */
    fprintf(finput,"%f\n",(*spectidx));
    /* Enter desired normalization at a=1: */
    fprintf(finput,"%f\n",-(*sigma_8));
    /* Enter 0,0 if you don't want the power spectrum written to a file  */
    fprintf(finput,"0,0\n");
    /* Enter dx (Mpc, =1 for scale-free), epsilon (Mpc, =0 for PM), etat (=0.05) */
    fprintf(finput,"1.0 0.0 0.05\n");
    /* Enter random number seed (9-digit integer)  */
    rseed[9]='\0';
    fprintf(finput,"%s\n",rseed);
    /* Enter ido (=1,2,3) for action: */
    fprintf(finput,"1\n");	
    fclose(finput);
    
    printf("(5).");
    step = 6;			/* Call the programm grafic */
    ret = system(buf); 
    if(ret<0 ||WEXITSTATUS(ret)==127 ) {error=6; break;}
    
    printf("(6).");
    step = 7;		        /* Checking the existence of delta.dat */
    finput=fopen("delta.dat","r");
    if(!finput) {error=7; break;}
    fclose(finput);

    printf("(7).");
    step = 8;		        /* Checking the existence of p3m.dat */
    finput=fopen("p3m.dat","r");
    if(!finput) {error=8; break;}
    fclose(finput);
    

    printf("(8).");
    /* Read the input */
    nparmax = (*np) * (*np) *(*np);
    initdm(x1,x2,x3,v1,v2,v3,
	   nx,ny,nz,&nparmax,
	   t0h0,at,vfact);
    
  }while(0);

  /* ******************** Cleaning Time ******************** */

  switch(step)
    {
    case 8:			/* Checking p3m.dat */
      if(error==8) perror("(8)Error with p3m.dat:");
      else if (remove("p3m.dat")==-1) perror("(8)Error removing p3m.dat file:");

    case 7:
      if(error==7) perror("(7)Error with delta.dat:");
      else if (remove("delta.dat")==-1) perror("(7)Error removing delta.dat file:");

    case 6:
      if(error==6) fprintf(stderr,"(6)Error calling grafic program\n   check if the executable is on the PATH.\n");
      
    case 5:
      if(error==5) perror("(5)Error with initial.in:");
      else if (remove("initial.in")==-1) perror("(5)Error removing initial.in file:");

    case 4:
      if(error==4) fprintf(stderr,"(4)Error creating executable name\n    check the np size.\n");

    case 3:
      if(error==3) perror("(3)Error changing working directory:");
      else if (chdir(oldwd)==-1) perror("(3)Error changing directory to old working directory:");

    case 2:
      if(error==2) fprintf(stderr,"(2)Error memorizing working directory\n");
      
    case 1:
      if(error==1) perror("(1)Error creating temporary directory:");
      else if (remove(tmpdir)==-1) perror("(1)Error removing temporary directory:");    
      break;
    }


  if(error>0) exit(1);
}
