/* Alloca multidimensional variable */


#include "global.h"
#include <stdio.h>
#include <stdlib.h>



#define ALLOCAIT(who,quantity,type) \
{ int size=0;\
  who = calloc(quantity,sizeof(type)); \
  if(quantity>1024) size = (quantity*sizeof(type))/1024; \
  if(!who) \
    { \
      fprintf(stderr,"\n Error memory allocation of %s\n",MKSTR(who)); \
      return -1; \
    } \
  gb->memuse  += size; \
}\
  printf(".")


#define DIVIDEALLOCAIT(who,number,quantity,type) \
{ int size=0,__i;\
  who = calloc(number,sizeof(type*)); \
  if(!who) \
    { \
      fprintf(stderr,"\n Error memory allocation of %s\n",MKSTR(who)); \
      return -1; \
    } \
  for(__i=0;__i<number;++__i){\
    who[__i] = calloc(quantity,sizeof(type)); \
    printf("D");\
    if(quantity>1024) size = (quantity*sizeof(type))/1024; \
    if(!who[__i]) \
      { \
        fprintf(stderr,"\n Error memory allocation of %s[%d]\n",MKSTR(who),__i); \
        return -1; \
      } \
  } \
  gb->memuse  += size; \
}\
  printf(".")

#define DIVIDEFREEIT(who,number) \
{ if(who){ int __i;\
    for(__i=0;__i<number;++__i){\
     if(who[__i]) free(who[__i]); \
    } \
    free(who);\
  }\
}\
  do{}while(0)



int allocagb(global *gb)
{
  int aksize;
  
  assert(gb);

  aksize = (gb->nx>gb->ny)?gb->nx:gb->ny;
  aksize = (aksize>gb->nz)?aksize:gb->nz;


  printf("\n Memory allocation:");
  /* Dark matter */

  

  ALLOCAIT(gb->x1,gb->nparmax,float);
  ALLOCAIT(gb->x2,gb->nparmax,float);
  ALLOCAIT(gb->x3,gb->nparmax,float);
  ALLOCAIT(gb->v1,gb->nparmax,float);
  ALLOCAIT(gb->v2,gb->nparmax,float);
  ALLOCAIT(gb->v3,gb->nparmax,float);


  /* Gravitationl  */
  ALLOCAIT(gb->gshape,gb->ntot,double);
  ALLOCAIT(gb->rhodm,gb->ntot,double);
  ALLOCAIT(gb->rhobm,gb->ntot,double);
  ALLOCAIT(gb->phi,gb->ntot,double);
  ALLOCAIT(gb->phiold,gb->ntot,double);
  ALLOCAIT(gb->gx,gb->ntot,double);
  ALLOCAIT(gb->gy,gb->ntot,double);
  ALLOCAIT(gb->gz,gb->ntot,double);
  ALLOCAIT(gb->gxold,gb->ntot,double);
  ALLOCAIT(gb->gyold,gb->ntot,double);
  ALLOCAIT(gb->gzold,gb->ntot,double);

  if(gb->egrav)
    {
      ALLOCAIT(gb->A,gb->ntot,FFTW_COMPLEX);
      ALLOCAIT(gb->ak,aksize,double);
      ALLOCAIT(gb->akq,aksize,double);
    }


  /* Baryonic */
  ALLOCAIT(gb->nes,gb->ntot,unsigned char);
  ALLOCAIT(gb->p,gb->ntot,double);
  ALLOCAIT(gb->vx,gb->ntot,double);
  ALLOCAIT(gb->vy,gb->ntot,double);
  ALLOCAIT(gb->vz,gb->ntot,double);
  if(gb->ebar)
    {
      ALLOCAIT(gb->pold,gb->ntot,double);
      ALLOCAIT(gb->rhobmold,gb->ntot,double);
      ALLOCAIT(gb->vxold,gb->ntot,double);
      ALLOCAIT(gb->vyold,gb->ntot,double);
      ALLOCAIT(gb->vzold,gb->ntot,double);
    }

  /* Baryonic divisions */

  if(gb->divide)
    {
      int dtot = gb->ntot/2;

      DIVIDEALLOCAIT(gb->dp,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->drhobm,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->drhobmold,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dphi,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dphiold,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dvx,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dvy,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dvz,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dvold,gb->ndiv,dtot,double);
      DIVIDEALLOCAIT(gb->dnes,gb->ndiv,dtot,unsigned char);
    }

  printf("ok\n");
  printf(" Total Memory allocation: %dMb\n\n",(int)gb->memuse/1024);

  return 0;
}


void freegb(global *gb)
{
  assert(gb);

  if(gb->x1) free(gb->x1);
  if(gb->x2) free(gb->x2);
  if(gb->x3) free(gb->x3);
  if(gb->v1) free(gb->v1);
  if(gb->v2) free(gb->v2);
  if(gb->v3) free(gb->v3);

  if(gb->gshape) free(gb->gshape);
  if(gb->rhodm) free(gb->rhodm);
  if(gb->rhobm) free(gb->rhobm);
  if(gb->phi) free(gb->phi);
  if(gb->phiold) free(gb->phiold);

  if(gb->egrav)
    {
      if(gb->A) free(gb->A);
      if(gb->ak) free(gb->ak);
      if(gb->akq) free(gb->akq);
    }

  if(gb->gx) free(gb->gx);
  if(gb->gy) free(gb->gy);
  if(gb->gz) free(gb->gz);
  
  if(gb->gxold) free(gb->gxold);
  if(gb->gyold) free(gb->gyold);
  if(gb->gzold) free(gb->gzold);

  if(gb->nes) free(gb->nes);
  if(gb->p) free(gb->p);
  if(gb->vx) free(gb->vx);
  if(gb->vy) free(gb->vy);
  if(gb->vz) free(gb->vz);

  if(gb->ebar)
    {
      if(gb->rhobmold) free(gb->rhobmold);
      if(gb->pold) free(gb->pold);
      if(gb->vxold) free(gb->vxold);
      if(gb->vyold) free(gb->vyold);
      if(gb->vzold) free(gb->vzold);
    }

  if(gb->divide)
    {
      { 
	if(gb->dp){ 
	  int __i;
	  for(__i=0;__i<gb->ndiv;++__i){
	    if(gb->dp[__i]) 
	      free(gb->dp[__i]); 
	  } 
	  free(gb->dp);
	}
      }  do{}while(0);
      //DIVIDEFREEIT(gb->dp,gb->ndiv); 
      DIVIDEFREEIT(gb->drhobm,gb->ndiv);
      DIVIDEFREEIT(gb->drhobmold,gb->ndiv);
      DIVIDEFREEIT(gb->dphi,gb->ndiv); 
      DIVIDEFREEIT(gb->dphiold,gb->ndiv);
      DIVIDEFREEIT(gb->dvx,gb->ndiv); 
      DIVIDEFREEIT(gb->dvy,gb->ndiv); 
      DIVIDEFREEIT(gb->dvz,gb->ndiv); 
      DIVIDEFREEIT(gb->dvold,gb->ndiv);
      DIVIDEFREEIT(gb->dnes,gb->ndiv); 
    }

}
