#include "global.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef MIN
#define MIN(n1,n2) (n1<n2)?n1:n2
#endif

#define STRNCMP(s1,s2) strncmp(s1,s2,MIN(strlen(s1),strlen(s2)))


#define NARGS 9




char *vargs[NARGS]=
  {
    "--nogrpc",
    "--egrav",
    "--ebar",
    "--input",
    "--notsc",
    "--nonblk",
    "--smart",
    "--nmap",
    "--cycles"
  };

char *vargsdesc[NARGS]=
  {
    "don't use GridSolve",
    "extend gravitational",
    "extend baryonic",
    "chose input file",
    "use faster CIC instead of slower triangular shaped clouds",
    "non blocking",
    "use SmartSolve when available",
    "number of smartsolve map",
    "number of cycles for smartsolve map"
  };

void arguments(global *gb, int argc, char *argv[])
{
  int i,j,k;

  // Default value
  
#ifdef HAVE_GRIDSOLVE
  gb->grpc = 1;
#else
  gb->grpc = 0;
#endif

  gb->smart=0;

  gb->egrav = 0;
  gb->edark = 0;
  gb->ebar  = 0;
  
  gb->ifile = "input.in";

  gb->tsc = 1;
  gb->divide=0;
  gb->ndiv=2;
  gb->nonblk=0;

  gb->cycles=3;
  gb->nmap=1;

  // Retreive args
  for(i=1;i<argc;i++)
    {
      j=0;
      while(j<NARGS && STRNCMP(vargs[j],argv[i])!=0)
	j++;

      switch(j)
	{
	case 0:			/* --nogrpc */
	  gb->grpc = 0;
	  break;
	case 1:			/* --exp-grav */
	  gb->egrav = 1;
	  break;
	case 2:			/* --exp-bar */
	  gb->ebar  = 1;
	  break;
	case 3:			/* --input */	  
	  i++;
	  if(i<argc)
	    {
	      gb->ifile=argv[i];
	    }
	  else
	    {
	      fprintf(stderr," Error: Missing 'filename' after argument '--input'\n");
	      fprintf(stderr," Example: --input input.in\n");
	      exit(1);
	    }
	  break;
	  
	case 4:			/* --notsc */
	  gb->tsc=0;
	  break;
	  
	case 5:			/* --nonblk */
	  gb->nonblk=1;
	  break;

	case 6:			/* --smart */
	  gb->smart=1;
	  break;

	case 7:			/* --map */	  
	  i++;
	  if(i<argc)
	    {
	      gb->nmap=atoi(argv[i]);
	    }
	  else
	    {
	      fprintf(stderr," Error: Missing 'number' after argument '--nmap'\n");
	      fprintf(stderr," Example: --nmap 1\n");
	      exit(1);
	    }
	  break;

	case 8:			/* --cycles */	  
	  i++;
	  if(i<argc)
	    {
	      gb->cycles=atoi(argv[i]);
	    }
	  else
	    {
	      fprintf(stderr," Error: Missing 'number' after argument '--cycles'\n");
	      fprintf(stderr," Example: --cycles 3\n");
	      exit(1);
	    }
	  break;

	default:
	  fprintf(stderr," Error: Unknown argument %s\n",argv[i]);
	  fprintf(stdout," Valid arguments:\n");
	  for(k=0;k<NARGS;k++)
	    {
	      fprintf(stderr," %s = %s\n",vargs[k],vargsdesc[k]);
	    }
	  exit(1);
	  break;
	}

      
    }



}
