#include "global.h"
#include "baryonic.h"
#include <stdlib.h>
#include <stdio.h>


void baryonic(global *gb)
{

  int side,lstep,dir;

  /* Search for shocked regions and  save t^n values of hydro variables */

  double *vold[3]={NULL,NULL,NULL};

  ETSTART(gb->etshsave);
  HPCALL(gb,searchsave,gb->nes,gb->p,gb->vx,gb->vy,gb->vz,gb->rhobm,
	 gb->pold,gb->vxold,gb->vyold,gb->vzold,gb->rhobmold,
	 &gb->nx,&gb->ny,&gb->nz,&gb->eta1);
  ETSTOP(gb->etshsave);

  /* Integrate hydro equations	*/

  lstep = gb->nsteps%6;

  vold[SLX]=gb->vxold;
  vold[SLY]=gb->vyold;
  vold[SLZ]=gb->vzold;

  for(side=0; side<3; side ++)
    {
      dir = gb->lflow[lstep][side];
      ETSTART(gb->etlxlylz);
      if(gb->divide)
	{
	  lxlylzdivide(gb,vold[dir],dir);
	}
      else
	{
	  HPCALL(gb,lxlylz,gb->p,gb->rhobm,gb->vx,gb->vy,gb->vz,
		 gb->phi,gb->phiold,gb->rhobmold,vold[dir],gb->nes,
		 &gb->nx,&gb->ny,&gb->nz,&gb->gamma,&gb->at,
		 &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
		 &gb->dmax,&gb->eta2,&gb->norm,&dir);
	}
      ETSTOP(gb->etlxlylz);
    }

  /* Compute the contribution of the expansion on the hydro variables
     calculate maximum velocity   */

  ETSTART(gb->etexspeed);
  HPCALL(gb,expandspeed,gb->p,gb->vx,gb->vy,gb->vz,gb->rhobm,
	 gb->pold,gb->vxold,gb->vyold,gb->vzold,
	 &gb->nx,&gb->ny,&gb->nz,&gb->dt,
	 &gb->dath,&gb->ath,&gb->gamma,&gb->bmvelmax);
  ETSTOP(gb->etexspeed);
}



void baryonic_nonblk(global *gb)
{

  taskvar tsearchsave;
  taskvar tlxlylz;
  taskvar texpandspeed;

  int side,lstep,dir;

  /* Search for shocked regions and  save t^n values of hydro variables */

  double *vold[3]={NULL,NULL,NULL};

  ETSTART(gb->etshsave);
  nonblockcall("searchsave",&tsearchsave,
	       gb->nes,gb->p,gb->vx,gb->vy,gb->vz,gb->rhobm,
	       gb->pold,gb->vxold,gb->vyold,gb->vzold,gb->rhobmold,
	       &gb->nx,&gb->ny,&gb->nz,&gb->eta1);


  waitnonblock(&tsearchsave);
  ETSTOP(gb->etshsave);

  /* Integrate hydro equations	*/

  lstep = gb->nsteps%6;

  vold[SLX]=gb->vxold;
  vold[SLY]=gb->vyold;
  vold[SLZ]=gb->vzold;

  for(side=0; side<3; side ++)
    {
      dir = gb->lflow[lstep][side];
      ETSTART(gb->etlxlylz);
      if(gb->divide)
	{
	  lxlylzdivide_nonblk(gb,vold[dir],dir);
	}
      else
	{
	  nonblockcall("lxlylz",&tlxlylz,
		       gb->p,gb->rhobm,gb->vx,gb->vy,gb->vz,
		       gb->phi,gb->phiold,gb->rhobmold,vold[dir],gb->nes,
		       &gb->nx,&gb->ny,&gb->nz,&gb->gamma,&gb->at,
		       &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
		       &gb->dmax,&gb->eta2,&gb->norm,&dir);
	  waitnonblock(&tlxlylz);
	}
      ETSTOP(gb->etlxlylz);
    }

  /* Compute the contribution of the expansion on the hydro variables
     calculate maximum velocity   */

  ETSTART(gb->etexspeed);
  nonblockcall("expandspeed",&texpandspeed,
	       gb->p,gb->vx,gb->vy,gb->vz,gb->rhobm,
	       gb->pold,gb->vxold,gb->vyold,gb->vzold,
	       &gb->nx,&gb->ny,&gb->nz,&gb->dt,
	       &gb->dath,&gb->ath,&gb->gamma,&gb->bmvelmax);

  waitnonblock(&texpandspeed);
  ETSTOP(gb->etexspeed);
}
