
#include <stdio.h>

#include "global.h"
#include "init.h"
#include "dark.h"
#include "baryonic.h"
#include "grav.h"



void printet(global *gb)
{
  printf(" EXECUTION TIME:\n");
  printf(" initialization  : %fs\n",ETAVG(gb->etinit));
  printf("   usegrafic       : %fs\n",ETAVG(gb->etgrafic));
  printf("   initbm          : %fs\n",ETAVG(gb->etinitbm));
  printf("   gravshape       : %fs\n",ETAVG(gb->etgravshape));
  printf("   initvel         : %fs\n",ETAVG(gb->etinitvel));
  printf(" evolve          : %fs\n",ETAVG(gb->etevolve));
  printf("   gravitational   : %fs\n",ETAVG(gb->etgrav));
  if(gb->egrav)
    {
      printf("     gravpot         : %fs\n",ETAVG(gb->etgravpot));
      printf("     gravforce       : %fs\n",ETAVG(gb->etgravforce));
      printf("     gravhalf        : %fs\n",ETAVG(gb->etgravhalf));
    }
  printf("   baryonic matter : %fs\n",ETAVG(gb->etbm));
  if(gb->ebar)
    {
      printf("     searchsave      : %fs\n",ETAVG(gb->etshsave));
      printf("     lxlylz*3        : %fs\n",ETAVG(gb->etlxlylz));
      printf("     expandspeed     : %fs\n",ETAVG(gb->etexspeed));
    }
  if(gb->tsc)
    {
      printf("   dark matter TSC : %fs\n",ETAVG(gb->etdm));
      printf("     densitydm  TSC  : %fs\n",ETAVG(gb->etdensitydm));
    }
  else
    {
      printf("   dark matter     : %fs\n",ETAVG(gb->etdm));
      printf("     densitydm       : %fs\n",ETAVG(gb->etdensitydm));
    }

  printf("\n");
  

}

void evolve_mainloop(global *gb)
{
  
  if(gs_smart_local_region()){
    printf("*********************\n  Evoltion step %d\n*********************\n\n",gb->nsteps );	    
    /* System evolution */
    timeinfo(&gb->at,&gb->atnew,
	   &gb->dat,&gb->datnew,
	   &gb->ath,&gb->dath,
	   &gb->t,&gb->told,
	   &gb->dt,&gb->dtold,
	   &gb->t0h0,&gb->hnow,&gb->nsteps);
  } 
	  
  if(gb->grpc && gb->nonblk)	/* Non blocking and GRPC */	    
    {
      evolve_nonblk(gb);
    }
  else
    {
      evolve(gb);
    }
  
  
  gb->nsteps++;		/* Increase the evolve step */


}


/* Evolve the system */

void evolve(global *gb)
{
  double ga;
  assert(gb);

  ETSTART(gb->etevolve);
  
  /* Calculate gravitational component */

#ifdef HAVE_SMARTSOLVE
  if(gs_smart_local_region())
    {
      ga = gb->gconst/gb->at;
    }
#else  /* GRIDSOLVE or LOCAL */
  ga = gb->gconst/gb->at;
#endif  


  ETSTART(gb->etgrav);
  if(gb->egrav)
    {
      gravitational(gb);
    }
  else
    {
      
      HPCALL(gb,fields,gb->phi,gb->phiold,gb->rhodm,gb->rhobm,
	     gb->gshape,gb->gx,gb->gy,gb->gz,gb->gxold,gb->gyold,gb->gzold,
	     &gb->nx,&gb->ny,&gb->nz,&ga,&gb->dt,&gb->dtold,&gb->nsteps);
    }
  ETSTOP(gb->etgrav);
  

  /* At the first step initialize the baryonic matter velocity field */

  if(gb->nsteps==0)
    {
      ETSTART(gb->etinitvel);
      HPCALL(gb,initvel,gb->phi,gb->vx,gb->vy,gb->vz,&gb->nx,&gb->ny,&gb->nz,
	     &gb->omega_dm,&gb->omega_bm,&gb->omega_v,&gb->at,&gb->dat);
      ETSTOP(gb->etinitvel);
    }
  
  /* Calculate baryonic matter component */

  ETSTART(gb->etbm);
  if(gb->ebar)
    {
      baryonic(gb);
    }
  else
    {
      HPCALL(gb,barmatter,gb->nes,gb->phi,gb->phiold,gb->p,gb->rhobm,
	     gb->vx,gb->vy,gb->vz,&gb->nx,&gb->ny,&gb->nz,&gb->at,&gb->dat,&gb->ath,
	     &gb->dath,&gb->dt,&gb->dtold,&gb->gamma,&gb->eta1,&gb->eta2,
	     &gb->dmax,&gb->norm,&gb->nsteps,(int *)gb->lflow,&gb->bmvelmax);
      
    }
  ETSTOP(gb->etbm);
  /* Calculate dark matter component */

  ETSTART(gb->etdm);
  if(gb->tsc)
    {
      HPCALL(gb,darkmattertsc,gb->x1,gb->x2,gb->x3,gb->v1,gb->v2,gb->v3,
	     gb->gx,gb->gy,gb->gz,gb->gxold,gb->gyold,gb->gzold,
	     gb->rhodm,&gb->nx,&gb->ny,&gb->nz,&gb->nparmax,
	     &gb->at,&gb->dat,&gb->ath,&gb->dath,&gb->dt,&gb->amass,
	     &gb->dmvelmax);
    }
  else
    {
      
      HPCALL(gb,darkmatter,gb->x1,gb->x2,gb->x3,gb->v1,gb->v2,gb->v3,
	     gb->gx,gb->gy,gb->gz,gb->gxold,gb->gyold,gb->gzold,
	     gb->rhodm,&gb->nx,&gb->ny,&gb->nz,&gb->nparmax,
	     &gb->at,&gb->dat,&gb->ath,&gb->dath,&gb->dt,&gb->amass,
	     &gb->dmvelmax);
    }
  ETSTOP(gb->etdm);

  /* Print execution time */

  ETSTOP(gb->etevolve);

#ifdef HAVE_SMARTSOLVE
  if(gs_smart_local_region())
#endif
    {
      printet(gb);

      /* Calculate new time step */
      
      timestep(&gb->dmvelmax,&gb->bmvelmax,&gb->at,&gb->atnew,
	       &gb->dat,&gb->datnew,&gb->ath,&gb->dath,
	       &gb->t,&gb->told,&gb->dt,&gb->dtold,&gb->omega_dm,
	       &gb->omega_bm,&gb->omega_v,&gb->t0h0,&gb->norm);
      
      gb->bmvelmax=0;
      gb->dmvelmax=0;
    }
}

void evolve_nonblk(global *gb)
{
#ifdef HAVE_GRIDSOLVE
  double ga;

  /* 
     Cause I'm using SmartSolve, the handle in taskvar cannot be destruct.
     So taskvar is allocated and not freed.     
  */

  taskvar *tinitvel;
  taskvar *tbarmatter;
  taskvar *tdarkmatter;

  /*TODO: better allocatation with checking */
  tbarmatter=(taskvar *)calloc(1, sizeof (taskvar));
  tdarkmatter=(taskvar *)calloc(1, sizeof (taskvar));
  
  tbarmatter->handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));
  tdarkmatter->handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));


  assert(gb);
  

  ETSTART(gb->etevolve);

  /* Calculate gravitational component */

  ga = gb->gconst/gb->at;

  ETSTART(gb->etgrav);
  if(gb->egrav)
    {
      // this function is not ready for SMART
      //gravitational_nonblk(gb);
    }
  //  else
    {
      blockcall("fields",
		gb->phi,gb->phiold,gb->rhodm,gb->rhobm,
		gb->gshape,gb->gx,gb->gy,gb->gz,gb->gxold,gb->gyold,gb->gzold,
		&gb->nx,&gb->ny,&gb->nz,&ga,&gb->dt,&gb->dtold,&gb->nsteps);
    }
  ETSTOP(gb->etgrav);



  /* At the first step initialize the baryonic matter velocity field */

  if(gb->nsteps==0)
    {
      ETSTART(gb->etinitvel);
      blockcall("initvel",
		gb->phi,gb->vx,gb->vy,gb->vz,&gb->nx,&gb->ny,&gb->nz,
		&gb->omega_dm,&gb->omega_bm,&gb->omega_v,&gb->at,&gb->dat);
      ETSTOP(gb->etinitvel);
    }


  /* Calculate baryonic matter component */

  ETSTART(gb->etbm);
  if(gb->ebar)
    {
      // this function is not ready for SMART
      //  baryonic_nonblk(gb);      
    }
  //else
    {
      nonblockcall("barmatter",tbarmatter,
		   gb->nes,gb->phi,gb->phiold,gb->p,gb->rhobm,
		   gb->vx,gb->vy,gb->vz,&gb->nx,&gb->ny,&gb->nz,&gb->at,&gb->dat,&gb->ath,
		   &gb->dath,&gb->dt,&gb->dtold,&gb->gamma,&gb->eta1,&gb->eta2,
		   &gb->dmax,&gb->norm,&gb->nsteps,(int *)gb->lflow,&gb->bmvelmax);      
    }



  /* Calculate dark matter component */

  ETSTART(gb->etdm);
  if(gb->tsc)
    {
      //nonblockcall("darkmattertsc",tdarkmatter,
      blockcall("darkmattertsc",
		   gb->x1,gb->x2,gb->x3,gb->v1,gb->v2,gb->v3,
		   gb->gx,gb->gy,gb->gz,gb->gxold,gb->gyold,gb->gzold,
		   gb->rhodm,&gb->nx,&gb->ny,&gb->nz,&gb->nparmax,
		   &gb->at,&gb->dat,&gb->ath,&gb->dath,&gb->dt,&gb->amass,
		   &gb->dmvelmax);
    }
  else
    {	 
      //nonblockcall("darkmatter",tdarkmatter,
      blockcall("darkmatter",
		   gb->x1,gb->x2,gb->x3,gb->v1,gb->v2,gb->v3,
		   gb->gx,gb->gy,gb->gz,gb->gxold,gb->gyold,gb->gzold,
		   gb->rhodm,&gb->nx,&gb->ny,&gb->nz,&gb->nparmax,
		   &gb->at,&gb->dat,&gb->ath,&gb->dath,&gb->dt,&gb->amass,
		   &gb->dmvelmax);
    }

#ifdef HAVE_SMARTSOLVE
  if(gs_smart_local_region())
#endif
    {
      //waitnonblock(tdarkmatter);
      ETSTOP(gb->etdm);
      
      if(!gb->ebar)
	waitnonblock(tbarmatter);
      ETSTOP(gb->etbm);
      
      /* Print execution time */
      ETSTOP(gb->etevolve);
      
      printet(gb);
      
      /* Calculate new time step */
      
      timestep(&gb->dmvelmax,&gb->bmvelmax,&gb->at,&gb->atnew,
	       &gb->dat,&gb->datnew,&gb->ath,&gb->dath,
	       &gb->t,&gb->told,&gb->dt,&gb->dtold,&gb->omega_dm,
	       &gb->omega_bm,&gb->omega_v,&gb->t0h0,&gb->norm);
      
      gb->bmvelmax=0;
      gb->dmvelmax=0;
    }

#ifndef HAVE_SMARTSOLVE		/* note the "not define" */
  /* GRIDOSLVE */ 
  /*TODO: better deallocatation with checking */
  free(tbarmatter->handle);
  free(tdarkmatter->handle);

  free(tbarmatter);
  free(tdarkmatter);

#endif 

#else  /* HAVE_GRIDSOLVE */

  fprintf(stderr,"GRPC not initializated, impossible to use non blocking call\n");

#endif


}




