#ifndef _GLOBAL_H_
#define _GLOBAL_H_ 1

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>

#include "fftw.h"

#ifdef HAVE_GRIDSOLVE
#undef HAVE_CONFIG_H
#include "grpc.h"
#define HAVE_CONFIG_H
#endif

#define SLX 0
#define SLY 1
#define SLZ 2

#define MKSTR(s) #s
#define CONCAT(a,b) a##b


#define HPCALL(gp,func,...) (gp->grpc)?blockcall(#func,__VA_ARGS__):func(__VA_ARGS__)


/*
#define HPCALL(gb,func,...) (gb->nonblk)?\
(HPNONBLKCALL(gb->grpc,func,__VA_ARGS__)):\
(HPBLKCALL(gb->grpc,func,__VA_ARGS__))
*/

#ifdef HAVE_SMARTSOLVE

#define ETINIT(test,et) do{ et.smart=test; if((!test) || gs_smart_local_region()){ et.c=0.0; et.t=0.0; et.n=0; } }while(0)
#define ETSTART(et)if((!et.smart) || gs_smart_local_region()){ et.c = cclock_(); }
#define ETSTOP(et) do{ if((!et.smart) || gs_smart_local_region()){ et.n+=1.0; et.t+= cclock_()-et.c; } }while(0)
#define ETAVG(et)  et.t/et.n 

#else  /* GRIDSOLVE or LOCAL */

#define ETINIT(test,et) do{et.smart=test; et.c=0.0; et.t=0.0; et.n=0; }while(0)
#define ETSTART(et) et.c = cclock_()
#define ETSTOP(et) do{et.n+=1.0; et.t+= cclock_()-et.c; }while(0)
#define ETAVG(et) et.t/et.n

#endif  






/* Structure to memorize execution time */

typedef struct exectime__
{
  double c;			/* Momorize the clock*/
  double t;			/* total time */
  double n;			/* Number of samples */
  int smart;			/* Use SmartGridSolve. ... default false!*/
}exectime;


/* Structure to memorize gridrpc task variable */

typedef struct taskvar__
{
#ifdef HAVE_GRIDSOLVE
  

  grpc_sessionid_t         sid;
  grpc_function_handle_t   *handle; /* it is a pointer for SmartSolve */

#endif

  int dummy;

}taskvar;

/* 
   Include the global variables used by the program,
   in a structure call gb.
 */


typedef struct _global
{
  long memuse;			/* Quantity of HEAP memory used by the program */

  char *ifile;			/* Input file */

  /* Arguments */

  int nonblk;			/* Do non blocking call */
  int grpc;			/* With Grid RPC ... default true!*/
  int smart;			/* Use SmartGridSolve. ... default false!*/
  int egrav;			/* Expand the gravitational function to fine grain functions*/
  int ebar;			/* Expand the baryonic function to fine grain functions*/
  int edark;			/* Expand the dark function to fine grain functions*/
  int tsc;			/* Use Triangular Shaped Cloud (TSC) interpolation*/
  int divide;	  		/* Divide the baryonic lxlylz in n working task  */
  int ndiv;			/* Number of division */
  int cycles;			/* Number of evolve cycles for map*/
  int nmap;		 	/* Number of map */

  /* Execution time for functions */

  exectime ettotal;		/* Execution time of total computation */

  exectime etinit;		/* Execution time initialization */
  exectime etgrafic;
  exectime etinitbm;
  exectime etinitvel;
  exectime etgravshape;
  exectime etbm;		/* Execution time baryonic matter */
  exectime etshsave;
  exectime etlxlylz;
  exectime etexspeed;
  exectime etdm;		/* Execution time dark matter */
  exectime etnbody;
  exectime etdensitydm;
  exectime etgrav;		/* Execution time gravitational */
  exectime etgravpot;
  exectime etgravforce;
  exectime etgravhalf;
  exectime etevolve;

  /* Task information */

  taskvar  initgrav;

  
  /* *********** Input parameters ********** */

  int np;			/* Numper of particles, nparmax = np^3 */
  int nx,ny,nz;			/* Grid sides size, X Y Z */
  double box;			/* Box size (Mpc/h) */
  double hnow;			/* Huble parameter */
  double omega_dm;		/* Omega dark matter */
  double omega_bm;		/* Omega baryonic matter */
  double omega_v;		/* Cosmological constant */
  double tempnow;		/* Present BM average temperature */
  double hfrac;			/* Hydrogen mass fraction */
  double spectidx;		/* Long-wave spectral index */
  double sigma_8;		/* Desired normalization */
  char rseed[10];	    	/* Random number seed (9-digit integer)  */

  /* *********** Multidimensional variables ********** */

  /* Gravitational */

  double *phi,*phiold;
  double *gshape;
  double *gx,*gy,*gz;
  double *gxold,*gyold,*gzold;
  FFTW_COMPLEX *A;
  double *ak,*akq;

  /* Dark matter */

  float *x1,*x2,*x3;
  float *v1,*v2,*v3;

  /* Baryonic matter */

  double *p,*pold;
  double *vx,*vy,*vz;
  double *vxold,*vyold,*vzold;
  unsigned char *nes;

  /* Baryonic matter divisions */

  double **dp;
  double **drhobm,**drhobmold;
  double **dphi,**dphiold;
  double **dvx,**dvy,**dvz;
  double **dvold;
  unsigned char **dnes;


  /* Density */

  double *rhodm;
  double *rhobm,*rhobmold;


  /* *********** Scalar variables ********** */

  /* Control flow variables */

  int lflow[6][3];

  /* Time related variables */

  int nsteps;
  double t0h0;
  double at,atnew;
  double dat,datnew;
  double ath,dath;
  double t,told;
  double dt,dtold;


  /* Density related variables */

  double amass,dmax;

  /* Fluid dynamics related variables */

  double eta1,eta2;
  double gamma;

  /* Normalization related variables */

  double vfact;			/* Constant to get velocities from km/sec to code units */
  double pfact;			/* Constant to get physical pressure from code pressure */

  /* Physical variables */

  double gconst;

  /* Speed related variables */

  double bmvelmax,dmvelmax;

  /* Grid related variables */

  int nparmax;			/* Number of particles max */
  int ntot;			/* Total number of grid points */
  double norm;

}global;


/* *********** Methods ********** */
double cclock_();		/* Read the CLOCK time */

void arguments(global *gb, int argc, char *argv[]); /* Process arguments */
void initscalar(global *);      /* Initialize the global scalar data */
void inittime(global *);        /* Initialize the global time data */
int allocagb(global *);         /* Alloca multidimensional variables */
void freegb(global *);		/* Free multidimensional variables */

void evolve_mainloop(global *gb);
void evolve(global *);		/* Evolve the system */
void evolve_nonblk(global *);		

void gravitational(global *);	/* Calculate the gravitational part*/
void gravitational_nonblk(global *gb);

void baryonic(global *);	/* Calculate the baryonic matter part*/
void baryonic_nonblk(global *);	

void lxlylzdivide(global *gb,double *vold, int dir); /* Divide the lxlylz in n task */
void lxlylzdivide_nonblk(global *gb,double *vold, int dir); 

void dark(global *);		/* Calculate the dark matter part*/


void initializedmbm(global *);	/* Initialize the darkmatter and baryonic components*/
void initializedmbm_nonblk(global *gb);

int initgrpc(global *gb);	/* Initialize grid rpc */
int closegrpc(global *gb);	/* Close gridrpc */

int blockcall(char *name, ...);	/* Grid RPC Block call */
int nonblockcall(char *name, taskvar *var,...);	/* Grid RPC Non blocking call */
int waitnonblock(taskvar *var);

/* Read inputs data */

#define indata  FC_FUNC(indata,INDATA)
#ifdef __cplusplus
extern "C" {
#endif

    extern void indata (int *, int *,int *, int *, 
			double *,double *,double *,double *,
			double *,double *,double *,double *,
			double *,char *,char *,int *);

#ifdef __cplusplus
}
#endif

/* Read inputs data */

#define s11abf  FC_FUNC(s11abf,S11ABF)
#ifdef __cplusplus
extern "C" {
#endif

  extern double s11abf (double *, int *);

#ifdef __cplusplus
}
#endif

/* Print information about loop variables  */

#define loopinfo  FC_FUNC(loopinfo,LOOOPINFO)
#ifdef __cplusplus
extern "C" {
#endif

  extern void loopinfo(double *,double *,double *,
		       double *,double *,double *,
		       double *,double *,double *,int *);

#ifdef __cplusplus
}

#endif

/* Don't know */

#define derivs  FC_FUNC(derivs,DERIVS)
#ifdef __cplusplus
extern "C" {
#endif

  extern double derivs(double *,double *, double *,double *);

#ifdef __cplusplus
}

#endif

/* Don't know*/

#define rk4  FC_FUNC(rk4,RK4)
#ifdef __cplusplus
extern "C" {
#endif

  extern double rk4(double *,double *,double *, double *,double *,double *, double *);

#ifdef __cplusplus
}

#endif

/* Calculate the new timestep */

#define timestep  FC_FUNC(timestep,TIMESTEP)
#ifdef __cplusplus
extern "C" {
#endif

  extern double timestep(double *,double *, double *, double *,double *,double *, 
			 double *,double *, double *, double *,double *,double *, 
			 double *,double *, double *, double *,double *);
			

#ifdef __cplusplus
}

#endif

/* Show info about time variables */

#define timeinfo  FC_FUNC(timeinfo,TIMESTEP)
#ifdef __cplusplus
extern "C" {
#endif

  extern double timeinfo(double *,double *, double *, double *,double *,double *, 
			 double *,double *, double *, double *,double *,double *, 
			 int *);

#ifdef __cplusplus
}

#endif

/* Check if an Halve is correct */

#define checkzhalve  FC_FUNC(checkzhalve,CHECKZHALVE)
#ifdef __cplusplus
extern "C" {
#endif

  extern double checkzhalve(double *,int *, int *, int *,
			    double *,double *, int *,char *);


#ifdef __cplusplus
}

#endif



#endif /* _GLOBAL_H */

