/* Grid rpc functions */

#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

/* va_copy is not portable, GNU work around */

#if !defined va_copy
#if defined __va_copy
#define va_copy __va_copy
#else
#define va_copy(dst, src) memcpy (&dst, &src, sizeof(va_list))
#endif
#endif



/* Grid RPC Block call */
/* Cause SmartGridSolve the handle is not going to be destructed */
/* TODO: keep a list of handle and destruct them when is apporpiate */

int blockcall(char *name, ...)
{
#ifdef HAVE_GRIDSOLVE

  va_list argptr;
  va_start(argptr,name);
  gs_va_list argstruct;

  int info;  
  
  grpc_function_handle_t *handle;
  handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));
  if(!handle)
    {
      fprintf(stderr,"error allocating handle memory");
      return -1;
    }


  /* 
     I'm going to set the "client" language to fortran 
     so GridSolve will consider all the arguments pass 
     by REFERENCE
   */

  if(grpc_set_client_language(GS_CALL_FROM_FORTRAN) != GRPC_NO_ERROR) 
    {
      grpc_perror("grpc_set_client_language");
      return -1;
    }

#ifdef HAVE_SMARTSOLVE
  if(grpc_function_handle_init(handle, "bind_servers_at_call_time", name) != GRPC_NO_ERROR) 
    {
      grpc_perror(name);
      return -1;
    }
#else  /* GRIDOSOLVE */
  if(grpc_function_handle_default(handle,name) != GRPC_NO_ERROR) 
    {
      grpc_perror(name);
      return -1;
    }
#endif


  va_copy(argstruct.args, argptr);

  info = grpc_call_valist_ft(handle, &argstruct);

  if(info != GRPC_NO_ERROR) 
    {
    grpc_perror(name);
    return -1;
    }

#ifndef HAVE_SMARTSOLVE		/* note the "not define" */
  /* GRIDOSLVE */
  grpc_function_handle_destruct(handle);
  free(handle);
#endif


#else

  fprintf(stderr,"GridSolve not initializated, impossible to call %s\n",name);

#endif

  return 0;
}




/* Grid RPC Non blocking call */

int nonblockcall(char *name, taskvar *var, ...)
{
#ifdef HAVE_GRIDSOLVE

  va_list argptr;
  va_start(argptr,var);
  gs_va_list argstruct;
  
  int info;


  /* 
     I'm going to set the "client" language to fortran 
     so GridSolve will consider all the arguments pass 
     by REFERENCE
   */

  if(grpc_set_client_language(GS_CALL_FROM_FORTRAN) != GRPC_NO_ERROR) 
    {
      grpc_perror("grpc_set_client_language");
      return -1;
    }


#ifdef HAVE_SMARTSOLVE
  if(grpc_function_handle_init(var->handle, "bind_servers_at_call_time", name) != GRPC_NO_ERROR) 
    {
      grpc_perror(name);
      return -1;
    }
#else  /* GRIDSOLVE */
  if(grpc_function_handle_default(&var->handle,name) != GRPC_NO_ERROR) 
    {
      grpc_perror(name);
      return -1;
    }
#endif


  va_copy(argstruct.args, argptr);

  info = grpc_call_valist_async_ft(var->handle,&var->sid,&argstruct);


  if(info != GRPC_NO_ERROR) 
    {
      grpc_perror(name);
      return -1;
    }

  
#else

  fprintf(stderr,"GridSolve not initializated, impossible to call %s\n",name);

#endif

  return 0;
}


int waitnonblock(taskvar *var)
{

#ifdef HAVE_GRIDSOLVE
  int status;
  
  if(!var)
    {
      fprintf(stderr,"internal error waitnonblock\n");
      return -1;
    }

  status=grpc_wait(var->sid);

  if(status != GRPC_NO_ERROR) {
    fprintf(stderr,"grpc_wait failed\n");
    return -1;
  }

#ifndef HAVE_SMARTSOLVE		/* note the "not define" */
  /* GRIDOSLVE */ 
  grpc_function_handle_destruct(var->handle);  
#endif 

#else

  fprintf(stderr,"GridSolve not initializated, impossible to wait\n");

#endif

  return 0;
}



int initgrpc(global *gb)
{

#ifdef HAVE_GRIDSOLVE
  /* Initialize gridrpc */
  if(grpc_initialize(NULL) != GRPC_NO_ERROR) 
    {
      grpc_perror("grpc_initialize");
      exit(EXIT_FAILURE);
    }


  printf("GRPC initialized\n");
#endif
  

  return 0;
}


int closegrpc(global *gb)
{

#ifdef HAVE_GRIDSOLVE
  /* Close gridrpc */
  grpc_finalize();
#endif

  
  return 0;
}


