#include "global.h"
#include "baryonic.h"
#include "init.h"
#include "dark.h"
#include "baryonic.h"
#include "grav.h"

#include <stdlib.h>
#include <stdio.h>

void initializedmbm(global *gb)
{

  /* Initialize dark matter*/

  ETSTART(gb->etgrafic);
  HPCALL(gb,usegrafic,gb->x1,gb->x2,gb->x3,gb->v1,gb->v2,gb->v3,
	    &gb->nx,&gb->ny,&gb->nz,&gb->np,&gb->t0h0,&gb->at,&gb->vfact,
	    &gb->omega_dm,&gb->omega_bm,&gb->omega_v,&gb->hnow, 
	    &gb->spectidx,&gb->sigma_8,gb->rseed);
  ETSTOP(gb->etgrafic);


  /* Initialize dark matter density */

  ETSTART(gb->etdensitydm);
  if(gb->tsc)
    {
      HPCALL(gb,densitydmtsc,gb->rhodm,gb->x1,gb->x2,gb->x3,
	     &gb->nx,&gb->ny,&gb->nz,&gb->nparmax,&gb->amass);
    }
  else
    {
      HPCALL(gb,densitydm,gb->rhodm,gb->x1,gb->x2,gb->x3,
	     &gb->nx,&gb->ny,&gb->nz,&gb->nparmax,&gb->amass);
    }
  ETSTOP(gb->etdensitydm);

  /* Initialize baryonic matter */
  ETSTART(gb->etinitbm);
  HPCALL(gb,initbm,gb->rhodm,gb->rhobm,gb->p,
	 &gb->nx,&gb->ny,&gb->nz,
	 &gb->at,&gb->pfact,&gb->omega_dm,&gb->omega_bm);
  ETSTOP(gb->etinitbm);


  /* Init grav */
  ETSTART(gb->etgravshape);
  HPCALL(gb,initgrav,gb->gshape,&gb->nx,&gb->ny,&gb->nz);
  ETSTOP(gb->etgravshape);


}


void initializedmbm_nonblk(global *gb)
{
#ifdef HAVE_GRIDSOLVE

  /* 
     Cause I'm using SmartSolve, the handle in taskvar cannot be destruct.
     So taskvar is allocated and not freed.     
  */

  taskvar *tinitgrav;
  taskvar *tusegrafic;
  taskvar *tdensity;
  taskvar *tinitbm;

  /*TODO: better allocatation with checking */
  tinitgrav=(taskvar *)calloc(1, sizeof (taskvar));
  tusegrafic=(taskvar *)calloc(1, sizeof (taskvar));
  tdensity=(taskvar *)calloc(1, sizeof (taskvar));
  tinitbm=(taskvar *)calloc(1, sizeof (taskvar));

  tinitgrav->handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));
  tusegrafic->handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));
  tdensity->handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));
  tinitbm->handle=(grpc_function_handle_t *)calloc(1, sizeof(grpc_function_handle_t));


  /* Init grav */
  ETSTART(gb->etgravshape);
  nonblockcall("initgrav",tinitgrav,
	       gb->gshape,&gb->nx,&gb->ny,&gb->nz);




  /* Initialize dark matter*/

  ETSTART(gb->etgrafic);
  blockcall("usegrafic",
	       gb->x1,gb->x2,gb->x3,gb->v1,gb->v2,gb->v3,
	       &gb->nx,&gb->ny,&gb->nz,&gb->np,&gb->t0h0,&gb->at,&gb->vfact,
	       &gb->omega_dm,&gb->omega_bm,&gb->omega_v,&gb->hnow, 
	       &gb->spectidx,&gb->sigma_8,gb->rseed);




  /* Initialize dark matter density */


  ETSTOP(gb->etgrafic);






  
  ETSTART(gb->etdensitydm);
  if(gb->tsc)
    {
      nonblockcall("densitydmtsc",tdensity,
		   gb->rhodm,gb->x1,gb->x2,gb->x3,
		   &gb->nx,&gb->ny,&gb->nz,&gb->nparmax,&gb->amass);
    }
  else
    {
      nonblockcall("densitydm",tdensity,
		   gb->rhodm,gb->x1,gb->x2,gb->x3,
		   &gb->nx,&gb->ny,&gb->nz,&gb->nparmax,&gb->amass);
    }

#ifdef HAVE_SMARTSOLVE
  if(gs_smart_local_region())
    {
      waitnonblock(tdensity);
    }
#else  /* GRIDSOLVE */
  waitnonblock(tdensity);
#endif


  ETSTOP(gb->etdensitydm);

  ETSTART(gb->etinitbm);
  nonblockcall("initbm",tinitbm,
	       gb->rhodm,gb->rhobm,gb->p,
	       &gb->nx,&gb->ny,&gb->nz,
	       &gb->at,&gb->pfact,&gb->omega_dm,&gb->omega_bm);

#ifdef HAVE_SMARTSOLVE
  if(gs_smart_local_region())
    {
      waitnonblock(tinitgrav);
    }
#else  /* GRIDSOLVE */
  waitnonblock(tinitgrav);
#endif
  ETSTOP(gb->etgravshape);
#ifdef HAVE_SMARTSOLVE
  if(gs_smart_local_region())
    {
      waitnonblock(tinitbm);
    }
#else  /* GRIDSOLVE */
  waitnonblock(tinitbm);
#endif
  ETSTOP(gb->etinitbm);




#ifndef HAVE_SMARTSOLVE		/* note the "not define" */
  /* GRIDOSLVE */ 
  /*TODO: better deallocatation with checking */
  free(tinitgrav->handle);
  free(tusegrafic->handle);
  free(tdensity->handle);
  free(tinitbm->handle);

  free(tinitgrav);
  free(tusegrafic);
  free(tdensity);
  free(tinitbm);
#endif 

#else  /* HAVE_GRIDSOLVE */

  fprintf(stderr,"GRPC not initializated, impossible to use non blocking call\n");

#endif


}
