#include "global.h"
#include "baryonic.h"
#include <stdlib.h>
#include <stdio.h>

/* Divide the grid in blocks (ndiv) ... compute the blocks ... merge the blocks */

void lxlylzdivide(global *gb,double *vold, int dir)
{
  int i;
  int nstart,nstop,step;

  int dnx,dny,dnz;

  switch(dir)
    {
    case 0:
    case 1:
      dnx = gb->nx; dny=gb->ny; dnz=gb->nz/gb->ndiv;
      step=dnz;
      break;
    case 2:
      dnx = gb->nx; dny=gb->ny/gb->ndiv; dnz=gb->nz;
      step=dny;
      break;

    }


  nstart=1;
  nstop=step;	    
  for(i=0;i<gb->ndiv;i++)	
    {	  	  
      HPCALL(gb,ldivide,gb->p,gb->rhobm,gb->vx,gb->vy,gb->vz,
	     gb->phi,gb->phiold,gb->rhobmold,vold,gb->nes,
	     &gb->nx,&gb->ny,&gb->nz,
	     gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
	     gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
	     &dnx,&dny,&dnz,&nstart,&nstop,&dir);
      nstart+=step;
      nstop+=step;
    }


  switch(dir)
    {
      /* ********************** X ******************* */
    case 0:            
      for(i=0;i<gb->ndiv;i++)
	HPCALL(gb,lx,gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
	       gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
	       &gb->nx,&gb->ny,&dnz,&gb->gamma,&gb->at,
	       &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
	       &gb->dmax,&gb->eta2,&gb->norm); 
      
      break;
      /* ********************** Y ******************* */
    case 1:
      for(i=0;i<gb->ndiv;i++)
	HPCALL(gb,ly,gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
	       gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
	       &gb->nx,&gb->ny,&dnz,&gb->gamma,&gb->at,
	       &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
	       &gb->dmax,&gb->eta2,&gb->norm); 
      break;
      /* ********************** Z ******************* */
    case 2:
      for(i=0;i<gb->ndiv;i++)
	HPCALL(gb,lz,gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
	       gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
	       &gb->nx,&dny,&gb->ny,&gb->gamma,&gb->at,
	       &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
	       &gb->dmax,&gb->eta2,&gb->norm); 
      break;
    }


  nstart=1;
  nstop=step;	    
  for(i=0;i<gb->ndiv;i++)
    {
      HPCALL(gb,lmerge,gb->p,gb->rhobm,gb->vx,gb->vy,gb->vz,gb->nes,
	     &gb->nx,&gb->ny,&gb->nz,
	     gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],gb->dnes[i],
	     &dnx,&dny,&dnz,&nstart,&nstop,&dir);
      nstart+=step;
      nstop+=step;
    }

  /*
  printf("CHECK Z HALVE START ... after computation\n");
  checkzhalve(gb->p,&gb->nx,&gb->ny,&gb->nz,gb->dp[0],gb->dp[1],&step,"p    ");
  printf("CHECK Z HALVE STOP ... after computation\n");
  */
}



void lxlylzdivide_nonblk(global *gb,double *vold, int dir)
{
  int i;
  int nstart,nstop,step;

  static taskvar *tdivide=NULL;
  static taskvar *tl=NULL;
  static taskvar *tmerge=NULL;

  int dnx,dny,dnz;

  if(tdivide==NULL)
    tdivide =(taskvar*)malloc(gb->ndiv*sizeof(taskvar));
  if(tl==NULL)
    tl =(taskvar*)malloc(gb->ndiv*sizeof(taskvar));
  if(tmerge==NULL)
    tmerge =(taskvar*)malloc(gb->ndiv*sizeof(taskvar));


  switch(dir)
    {
    case 0:
    case 1:
      dnx = gb->nx; dny=gb->ny; dnz=gb->nz/gb->ndiv;
      step=dnz;
      break;
    case 2:
      dnx = gb->nx; dny=gb->ny/gb->ndiv; dnz=gb->nz;
      step=dny;
      break;

    }


  nstart=1;
  nstop=step;	    
  for(i=0;i<gb->ndiv;i++)	
    {	  	  
      nonblockcall("ldivide",&tdivide[i],
		   gb->p,gb->rhobm,gb->vx,gb->vy,gb->vz,
		   gb->phi,gb->phiold,gb->rhobmold,vold,gb->nes,
		   &gb->nx,&gb->ny,&gb->nz,
		   gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
		   gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
		   &dnx,&dny,&dnz,&nstart,&nstop,&dir);
      nstart+=step;
      nstop+=step;
    }

  for(i=0;i<gb->ndiv;i++)
    {
      waitnonblock(&tdivide[i]);
    }	


  switch(dir)
    {
      /* ********************** X ******************* */
    case 0:            
      for(i=0;i<gb->ndiv;i++)
	nonblockcall("lx",&tl[i],
		     gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
		     gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
		     &gb->nx,&gb->ny,&dnz,&gb->gamma,&gb->at,
		     &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
		     &gb->dmax,&gb->eta2,&gb->norm); 
      
      break;
      /* ********************** Y ******************* */
    case 1:
      for(i=0;i<gb->ndiv;i++)
	nonblockcall("ly",&tl[i],
		     gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
		     gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
		     &gb->nx,&gb->ny,&dnz,&gb->gamma,&gb->at,
		     &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
		     &gb->dmax,&gb->eta2,&gb->norm); 
      break;
      /* ********************** Z ******************* */
    case 2:
      for(i=0;i<gb->ndiv;i++)
	nonblockcall("lx",&tl[i],
		     gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],
		     gb->dphi[i],gb->dphiold[i],gb->drhobmold[i],gb->dvold[i],gb->dnes[i],
		     &gb->nx,&dny,&gb->ny,&gb->gamma,&gb->at,
		     &gb->ath,&gb->dat,&gb->dt,&gb->dtold,
		     &gb->dmax,&gb->eta2,&gb->norm); 
      break;
    }

  for(i=0;i<gb->ndiv;i++)
    {
      waitnonblock(&tl[i]);
    }	

  nstart=1;
  nstop=step;	    
  for(i=0;i<gb->ndiv;i++)
    {
      nonblockcall("lmerge",&tmerge[i],
		   gb->p,gb->rhobm,gb->vx,gb->vy,gb->vz,gb->nes,
		   &gb->nx,&gb->ny,&gb->nz,
		   gb->dp[i],gb->drhobm[i],gb->dvx[i],gb->dvy[i],gb->dvz[i],gb->dnes[i],
		   &dnx,&dny,&dnz,&nstart,&nstop,&dir);
      nstart+=step;
      nstop+=step;
      waitnonblock(&tmerge[i]);
    }



  /*
  printf("CHECK Z HALVE START ... after computation\n");
  checkzhalve(gb->p,&gb->nx,&gb->ny,&gb->nz,gb->dp[0],gb->dp[1],&step,"p    ");
  printf("CHECK Z HALVE STOP ... after computation\n");
  */
}
