#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "global.h"
#include "init.h"
#include "grpc.h"
#include "gs_smart_clib.h"
int main(int argc, char *argv[])
{
  int cycles_index,nmap_index,length;
  int nstepold;

  /* Create global data */

  global gb;

  /* Process arguments */

  arguments(&gb,argc,argv);

  /* Read input parameters */
  
  length = strlen(gb.ifile);
  indata(&gb.np,&gb.nx,&gb.ny,&gb.nz,&gb.box,&gb.hnow,
	 &gb.omega_dm, &gb.omega_bm,&gb.omega_v,&gb.tempnow,
	 &gb.hfrac,&gb.spectidx,&gb.sigma_8,gb.rseed,
	 gb.ifile,&length);

  
  /* Initialize global scalar variable */

  initscalar(&gb);

  /* Alloca memory*/

  allocagb(&gb);

  /* Initialize gridrpc */

  if(gb.grpc)
    initgrpc(&gb);

  printf("*********************  Total steps %d  *********************\n\n",gb.nmap*gb.cycles );	          


  gb.nsteps=0;
    
  nmap_index=gb.nmap;
          
    /* Initialize time value */
  ETINIT(gb.smart,gb.ettotal);
  ETINIT(gb.smart,gb.etinit);
  ETINIT(gb.smart,gb.etinitvel);	
  ETINIT(gb.smart,gb.etgrafic);
  ETINIT(gb.smart,gb.etinitbm);
  ETINIT(gb.smart,gb.etgravshape);
  ETINIT(gb.smart,gb.etbm);	
  ETINIT(gb.smart,gb.etshsave);
  ETINIT(gb.smart,gb.etlxlylz);
  ETINIT(gb.smart,gb.etexspeed);
  ETINIT(gb.smart,gb.etdm);	
  ETINIT(gb.smart,gb.etnbody);
  ETINIT(gb.smart,gb.etdensitydm);
  ETINIT(gb.smart,gb.etgrav);	
  ETINIT(gb.smart,gb.etgravpot);
  ETINIT(gb.smart,gb.etgravforce);
  ETINIT(gb.smart,gb.etgravhalf);
  ETINIT(gb.smart,gb.etevolve);
    
  /* Start timing the total ammount */
  ETSTART(gb.ettotal);
    
  /* Initialize dark and baryonic matter*/
  ETSTART(gb.etinit);
  if(gb.grpc && gb.nonblk)	
  {     
    initializedmbm_nonblk(&gb);  
  }
  else
  {
    initializedmbm(&gb);  	
  }
    
  ETSTOP(gb.etinit);

  /* Initialize time parameters */
  inittime(&gb);
    
  nstepold=gb.nsteps;
  nstepold=0;
   
  GS_SMART_MAP("", "")
  {
    gb.nsteps=nstepold; /* 1 cycle same value - 2 cycle restore value*/

    cycles_index=gb.cycles;	       
	      
    for(;cycles_index>0;--cycles_index) /* Number of cycles */
    {
      evolve_mainloop(&gb);
    }
  }
  /* Free memory */
  ETSTOP(gb.ettotal);

  printf("\n          RUN ON : ");
  if(gb.grpc)
  {
    if(gb.smart)
      printf("SmartSolve.\n");
    else
      printf("GridSolve.\n");
  }
  else
  {
    printf("local computer.\n");
  }

  printf(" TOTAL EXEC TIME : %fs\n\n",ETAVG(gb.ettotal));

  
  freegb(&gb);


  closegrpc(&gb);

  return 0;
}
