!     
      SUBROUTINE timestep(dmvelmax,bmvelmax,
     .     at,atnew,dat,datnew,ath,dath,
     .     t,told,dt,dtold,omega_dm,omega_bm,
     .     omega_v,t0h0,norm)

      IMPLICIT NONE

!
! parameters
!
	
      REAL*8 dmvelmax,bmvelmax
      REAL*8 at,atnew,dat,datnew,ath,dath
      REAL*8 t,told,dt,dtold,omega_dm,omega_bm
      REAL*8 omega_v,t0h0,norm

!     
! local variables
!
      REAL*8, PARAMETER :: dt0=0.000000001
      REAL*8, PARAMETER :: ca=0.020
      REAL*8, PARAMETER :: cour=0.9
      REAL*8, PARAMETER :: tstop=1.0

      REAL*8 :: rdtc,dta,dth,thalf
      REAL*8 :: dx,omega_m
      REAL*8 :: velmax
      REAL*8 :: derivs,rk4
!     
! calculate quantities at t^n
!

      omega_m=omega_dm+omega_bm
      dx=1.0*norm

      told=t
      dtold=dt
!     
      if(omega_m.eq.1.0) then
         at=t**(2.0/3.0)
         dat=(2.0/3.0)*t**(-1.0/3.0)
         dta=((1.0+ca)*at)**(3.0/2.0)-t
      else
         dat=datnew
         at=atnew
         dta=ca*at/dat
      end if

!
! calculate new timestep
!
      velmax=max(dmvelmax,bmvelmax)

      rdtc=velmax/(cour*dx*at)
      dt=1.0/rdtc

      dt=min(dt,dta)
!      
      if(dt.eq.dt0)write(*,*)'dt = dt0'
      
      t=t+dt
      if(t.gt.tstop)then
         t=tstop
         dt=t-told
      endif
      dth=0.5*dt
      thalf=told+dth
!     
!     calculate quantities at t^{n+1} and t^{n+1/2}
!     
      if(omega_m.eq.1.0) then
         atnew=t**(2.0/3.0)
         ath=thalf**(2.0/3.0)
         dath=(2.0/3.0)*thalf**(-1.0/3.0)
         datnew=(2.0/3.0)*t**(-1.0/3.0)
      else

         atnew=rk4(at,dat,told,dt,t0h0,omega_m,omega_v)
         ath=rk4(at,dat,told,dth,t0h0,omega_m,omega_v)
         
!     
         datnew=derivs(atnew,t0h0,omega_m,omega_v)
         dath=derivs(ath,t0h0,omega_m,omega_v)
!     
      endif
!     
      write(*,*)''
      write(*,99)'Maximum DM velocity: ',dmvelmax
      write(*,99)'Maximum BM velocity: ',bmvelmax
      write(*,*)''
      write(*,97)'****************************************************'

 99   format(1x,a31,3(1x,e13.7))
 97   format(1x,a55)        
      END SUBROUTINE timestep
