/*
 * @file
 *
 * SmartGridSolve client library
 */

#include "config.h"
#ifndef GS_SMART_CLIB
#define GS_SMART_CLIB

/*
 * Main mapping function for SmartGridSolve. 
 * USAGE:
 * while(gs_smart_map("mapper_name")){
 *   ..
 *   // group of tasks to map
 *   ..
 *  }
 * 
 */

#define gs_smart_map gs_smart_map_two_cycles

#define GS_SMART_MAP(...) while(gs_smart_map_two_cycles(__VA_ARGS__))

/*
 * Function maps a group of tasks by iterating through
 * the group of tasks twice.  First cycle the 
 * task graph is generated. At the beginning of the second
 * cycle the group of tasks is mapped. On the second cycle
 * the group of tasks is mapping according to the mapping 
 * solution
 */
int gs_smart_map_two_cycles(char * mapper_name, char * comm_type);

/*
 * This function is used when there is section of code that should 
 * be ignored during the first cycle when the the task graph is in 
 * the process of being built.
 *
 * USAGE:
 *
 * if(gs_smart_local_region()){
 *  ..
 *  //section of code to ignore during task graph building
 *  ..
 * }
 */

int gs_smart_local_region();

#endif 
