#include <termios.h>
#include <stdio.h>
#include <string.h>

#include "utility.h"

/**
 * Turns off echo to the terminal and gets a password from the
 * user.  Resets the terminal before returning.
 *
 * @param prompt -- The prompt that you would like displayed before
 *   reading the password (e.g. "Password:").  If NULL, no prompt is
 *   displayed.
 * 
 * @returns allocated string containing the password entered.
 */

char *
gs_get_password(char *prompt)
{
  struct termios initialrsettings, newrsettings;
  char *password;

  if(prompt)
    printf("%s", prompt);

  tcgetattr(fileno(stdin), &initialrsettings);
  newrsettings = initialrsettings;
  newrsettings.c_lflag &= ~ECHO;

  if(tcsetattr(fileno(stdin), TCSAFLUSH, &newrsettings) != 0)
    return NULL;
  else {
    password = gs_get_line(stdin);

    tcsetattr(fileno(stdin), TCSANOW, &initialrsettings);

    if(!password)
      return NULL;

    if(strlen(password) > 0)
      password[strlen(password)-1] = 0;
  }

  printf("\n");

  return password;
}
