/**
 * @file
 *
 * This program will ask the agent to terminate.  The usage is:
 *    GS_killagent <agent name>
 *
 */
/* $Id: gs_kill_agent.c,v 1.21 2007/04/11 16:47:34 seymour Exp $ */
/* $UTK_Copyright: $ */

#include "utility.h"

#include <stdlib.h>
#include <string.h>

#include "comm_data.h"
#include "comm_basics.h"
#include "general.h"

char * gs_get_password(char *prompt);

/**
 * Entry point for GS_killagent.
 *
 * @param argc -- arg count
 * @param argv -- array of command line args
 *
 * @returns EXIT_SUCCESS on success, EXIT_FAILURE on error.
 */

int
main(int argc, char **argv)
{
  int sock, tag, agentport;
  char *password;

  if(argc < 2) {
    fprintf(stderr,"Usage: GS_killagent <agent name>\n");
    exit(EXIT_FAILURE);
  }

  agentport = getenv_int("GRIDSOLVE_AGENT_PORT", GRIDSOLVE_AGENT_PORT_DEFAULT);
  if((sock = gs_connect_direct(argv[1], agentport)) == INVALID_SOCKET) {
    fprintf(stderr,"%s cannot be contacted\n", argv[1]);
    exit(EXIT_FAILURE);
  }

  if((gs_send_tag(sock, GS_PROT_KILL_AGENT) < 0) ||
     (gs_send_string(sock, VERSION) < 0)) {
    fprintf(stderr,"Error communicating with agent.\n");
    exit(EXIT_FAILURE);
  }

  if(gs_recv_tag(sock, &tag) < 0) {
    fprintf(stderr,"Error communicating with agent.\n");
    exit(EXIT_FAILURE);
  }

  if(tag != GS_PROT_OK) {
    if(tag == GS_PROT_VERSION_MISMATCH)
      fprintf(stderr, "Error: Agent is an incompatible version\n");
    else
      fprintf(stderr, "Error: Agent refused with code %d\n", tag);
    exit(EXIT_FAILURE);
  }

  password = gs_get_password("Password: ");

  if(!password) {
    fprintf(stderr,"Error reading password.\n");
    exit(EXIT_FAILURE);
  }

  if(gs_send_string(sock, password) < 0) {
    fprintf(stderr,"Error sending password\n");
    exit(EXIT_FAILURE);
  }

  if(gs_recv_tag(sock, &tag) < 0) {
    fprintf(stderr,"Error receving confirmation tag\n");
    exit(EXIT_FAILURE);
  }

  if(tag != GS_PROT_OK) {
    fprintf(stderr, "Not Killed, error code = %d\n", tag);
    exit(EXIT_FAILURE);
  }

  printf("Successfully Killed\n");

  exit(EXIT_SUCCESS);
}
