cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	program grafic
c  Generate initial conditions for cosmological N-body integration
c  as a Gaussian random field.
c
c
c	parameter (np1=64,np2=64,np3=64,ncon=1)
c
c	parameter (npart=np1*np2*np3)
c
	integer npinit,error
	integer np1,np2,np3
	integer npart
	REAL,DIMENSION (:), ALLOCATABLE :: delta
	REAL,DIMENSION (:,:), ALLOCATABLE :: v,x
c	real x(3,npart),v(3,npart),delta(npart)
	double precision sigma,sigd,ekin0
	real omegabm
	real(kind=4) x1r4,x2r4,x3r4,v1r4,v2r4,v3r4
	real xxn,yyn,zzn
	common /cosmoparms/ omegam,omegav,h0
        common/baryons/omegabm
c	common/mio/v,delta
c	common/mio1/x
c
c  npart is the number of particles; initial conditions give displacements
c  from a grid of size np1*np2*np3=npart.  dx is the mean interparticle
c  spacing in comoving Mpc; omegam, omegav, and h0 are at a=1.
c  Initial positions and velocities are provided at a=astart<1.
c
	write(*,*) 'Enter particle lattice size: np='
	read(*,*) npinit
	np1=npinit
	np2=npinit
	np3=npinit
	npart=np1*np2*np3
	ncon=1
	allocate(delta(npart),STAT=error)
        allocate(v(3,npart),STAT=error)
        allocate(x(3,npart),STAT=error)

	xxn=float(np1)
	yyn=float(np2)
	zzn=float(np3)
c  Initialize power spectrum.
	call pini
c

c
	write(*,*) 'Enter dx (Mpc, =1 for scale-free), epsilon (Mpc,',
     &             ' =0 for PM), etat (=0.05)'
	read(*,*) dx,epsilon,etat
	write(*,1001) 'npart, L_x, L_y, L_z=',npart,np1*dx,np2*dx,
     2             np3*dx,' Mpc'
1001	format(1x,a21,i9,3(2x,f7.2),a4)
	amass0=2.776e7*omegam*h0**2*dx**3
	write(*,1002) 'Particle mass= ',amass0,' solar masses'
1002	format(1x,a15,e9.4,a13)
c  Choose your own random number seed.
	write(*,*) 'Enter random number seed (9-digit integer)'
	read(*,*) iseed
	call randini(iseed)
	write(*,*) 'Enter ido (=1,2,3) for action:'
	write(*,*) '  ido=1 Compute unconstrained realization'
	write(*,*) '  ido=2 Compute mean field of constraints'
	write(*,*) '  ido=3 Compute constrained realization'
	read(*,*) ido
	if (ido.lt.1.or.ido.gt.3) stop
	if (ido.gt.1) then
	  write(*,*)
     2 'Make sure you have provided constraints in subroutine constr'
	end if
c  Compute realizations of density perturbation and displacement fields.
	call ic3(v,delta,ido,dx,delm,dispm,np1,np2,np3,npart)
c  Scale density and displacement back to starting redshift.
c  Require that max |delta_rho/rho| <= delmax at a=astart.
	delmax=1.0
	dpl=delmax/delm
cc  Require that max displacement <= deltax at a=astart.
c	dpl=dx/dispm
c  No, actually scale it so that maximum delta=1 at a=astart.
c  To change it to displacement scaling, uncomment the previous line.
	dpls=dpl*dplus(1.0,omegam,omegav)
	astart=adp(dpls,omegam,omegav)
	write(*,*)
	write(*,*) 'Scaling density and displacements to a=',astart
	xfact=dpl/dx
	vfact=xfact*fomega(astart,omegam,omegav)*
     2        dladt(astart,omegam,omegav)/astart
c
	sigma=0.0
	delm=0.0
	sigd=0.0
	dispm=0.0
	ekin0=0.0
	j=0
	  do 30 j3=1,np3
	  x3=(j3-1)
	    do 20 j2=1,np2
	    x2=(j2-1)
	      do 10 j1=1,np1
	      x1=(j1-1)
	      j=j+1
c  Compute statistics.
	      sigma=sigma+delta(j)**2
	      delm=max(delm,abs(delta(j)))
	      dsigd=v(1,j)**2+v(2,j)**2+v(3,j)**2
	      sigd=sigd+dsigd
	      dispm=max(dispm,dsigd)
c  Rescale density perturbation.
	      delta(j)=delta(j)*dpl
c  Apply displacements.
	      x(1,j)=x1+v(1,j)*xfact
	      x(2,j)=x2+v(2,j)*xfact
	      x(3,j)=x3+v(3,j)*xfact
c  Periodic boundary conditions.
	      if (x(1,j).lt.0.0) x(1,j)=x(1,j)+xxn
	      if (x(1,j).ge.xxn) x(1,j)=x(1,j)-xxn
	      if (x(2,j).lt.0.0) x(2,j)=x(2,j)+yyn
	      if (x(2,j).ge.yyn) x(2,j)=x(2,j)-yyn
	      if (x(3,j).lt.0.0) x(3,j)=x(3,j)+zzn
	      if (x(3,j).ge.zzn) x(3,j)=x(3,j)-zzn
c  Compute velocity.
	      v(1,j)=v(1,j)*vfact
	      v(2,j)=v(2,j)*vfact
	      v(3,j)=v(3,j)*vfact
c  Compute kinetic energy.
	      ekin0=ekin0+v(1,j)**2+v(2,j)**2+v(3,j)**2
10	    continue
20	  continue
30	continue
	sigma=sqrt(sigma/npart)*dpl
	delm=delm*dpl
	sigd=sqrt(sigd/npart)*dpl
	dispm=sqrt(dispm)*dpl
	write(*,*)
	write(*,*) 'For a=astart: linear sigma, delmax=',real(sigma),delm
	write(*,*) 'RMS, max. 3-D displacement=',real(sigd),
     2             dispm,' Mpc'
c  Set parameters for output p3m file.
	nstep=0
c  First timestep: max. displacement=epsilon (or 0.5*dx for PM).
c  Note that dt := H0*dtau/a where dtau is conformal time -- this t is
c  not proper time!
	dterm=fomega(astart,omegam,omegav)*dladt(astart,omegam,omegav)
	if (epsilon.gt.0.0) then
	  dt=(epsilon/dispm)/dterm
	else
	  dt=(0.5*dx/dispm)/dterm
	end if
c  Cosmological timestep criterion: limit timestep to 0.1*Hubble time.
	dterm=0.1/dladt(astart,omegam,omegav)
	dt=min(dt,dterm)
c  Proper specific energies in units of (km/s)**2.
	ekin=0.5*ekin0/npart*(h0*dx)**2
	egrav=-1.5*ekin
	egint=egrav/3.0
c
c  Output delta.dat file.

	open(10,file='delta.dat',access='direct',recl=4)
	x1r4=real(astart)
	write(10,rec=1)x1r4
	x1r4=real(dx)
	write(10,rec=2)x1r4
	do j=1,npart
	  x1r4=real(delta(j))
	  write(10,rec=j+2)x1r4
	enddo
	close(10)

c  Output p3m.dat file.

	open(10,file='p3m.dat',access='direct',recl=24)

C
C position are in units of grid while velocities are dx/dt where dx is
C in grid units and dt is in H_0 units
C In order to use this result with hydrompi velocities must be recalculated
C as v_hydro = v * H0 * t0 (H0 * t0 is given by equation 17 of Carrol
C Press and Turner)
C

	do j=1,npart
	  x1r4=real(x(1,j))
	  x2r4=real(x(2,j))
	  x3r4=real(x(3,j))
	  v1r4=real(v(1,j))
	  v2r4=real(v(2,j))
	  v3r4=real(v(3,j))
C	
	  write(10,rec=j)x1r4,x2r4,x3r4,v1r4,v2r4,v3r4
	enddo
	close(10)
c
	stop

	deallocate(delta)
        deallocate(v)
        deallocate(x)

	end
